from gryphon.lib.money import Money

from collections import defaultdict

class Position(dict):
    def __missing__(self, key):
        value = self[key] = Money(0, key)
        return value

class Brain(object):
    def __init__(self):
        pass

    def __repr__(self):
        return str(self)
    def __str__(self):
        return str(self.__dict__)



# -*- coding: utf-8 -*-

from gryphon.lib.exchange import exceptions


ALL_EXCHANGE_KEYS = [
    'bitstamp_btc_usd',
    'bitfinex_btc_usd',
    'kraken_btc_eur',
    'kraken_btc_usd',
    'kraken_btc_cad',
    'itbit_btc_usd',
    'okcoin_btc_usd',
    'coinbase_btc_usd',
    'quadriga_btc_cad',
    'gemini_btc_usd',
]

HISTORICAL_EXCHANGE_KEYS = [
    'vaultofsatoshi_btc_cad',
    'bitme_btc_usd',
    'buttercoin_btc_usd',
    'cavirtex_btc_cad',
    'coinsetter_btc_usd',
]

BANK_ACCOUNT_KEYS = ['BMO_USD', 'BMO_CAD', 'BMO_CAD_OPS', 'BOA_MAIN', 'BOA_INCOME']


def all_exchanges():
    return [make_exchange_from_key(key) for key in ALL_EXCHANGE_KEYS]


def all_exchange_datas(db):
    return make_exchange_datas_from_keys(ALL_EXCHANGE_KEYS, db)

def all_bank_accounts(db):
    return make_exchange_datas_from_keys(BANK_ACCOUNT_KEYS, db)

def historical_exchanges():
    return [make_exchange_from_key(key) for key in HISTORICAL_EXCHANGE_KEYS]


def historical_exchange_datas(db):
    return make_exchange_datas_from_keys(HISTORICAL_EXCHANGE_KEYS, db)


def all_current_and_historical_exchanges():
    current_exchanges = all_exchanges()
    current_exchanges.extend(historical_exchanges())

    return current_exchanges


def canonical_key(key):
    key = key.upper()
    if key == 'VAULT':
        key = 'VAULTOFSATOSHI'
    if key == 'BUTTER':
        key = 'BUTTERCOIN'

    return key


def map_pair_name_to_exchange_name(pair_name):
    """
    We're preparing to add the notion that exchanges can have multiple trading pairs
    into our system. Each exchange is going to have a single ExchangeData db object but
    have one wrapper for each pair. Order.exchange_name is going to refer to the pair,
    but most accounting functions take place on the ExchangeData object. Thus, we need
    a mapping of ExchangeWrapper -> ExchangeData. This function will serve that purpose
    for now.
    """

    if pair_name == 'GEMINI_ETH_USD':
        return 'GEMINI_BTC_USD'
    else:
        return pair_name


def make_exchange_from_key(key):
    key = canonical_key(key)

    api_wrapper_class = get_api_wrapper_class_by_name(key)

    return api_wrapper_class()


def make_exchange_data_from_key(key, db):
    keys = [key]
    exchange_datas = make_exchange_datas_from_keys(keys, db)

    assert len(exchange_datas) == 1

    return exchange_datas[0]


def initialized_ledgers(db):
    """
    Give us the names of the exchanges that have initialized ledgers in our trading
    database.
    """
    from gryphon.lib.models.exchange import Exchange as ExchangeData

    exchange_account_names = db.query(ExchangeData.name).all()
    exchange_account_names = [e[0] for e in exchange_account_names]

    return exchange_account_names


def get_all_initialized_exchange_wrappers(db):
    from gryphon.lib.models.exchange import Exchange as ExchangeData

    exchange_accounts = db.query(ExchangeData).all()

    exchange_wrappers = [
        make_exchange_from_key(e.name) for e in exchange_accounts
        if e.name.lower() in ALL_EXCHANGE_KEYS
    ]

    return exchange_wrappers


def make_exchange_datas_from_keys(pair_names, db):
    from gryphon.lib.models.exchange import Exchange as ExchangeData

    canonical_pair_names = [canonical_key(k) for k in pair_names]
    exchange_names = [map_pair_name_to_exchange_name(p) for p in canonical_pair_names]

    exchange_datas = db.query(ExchangeData)\
        .filter(ExchangeData.name.in_(exchange_names))\
        .all()

    assert len(exchange_datas) == len(pair_names)

    return [exchange_datas[0]]


def get_api_wrapper_class_by_name(exchange_name):
    exchange_name = canonical_key(exchange_name)

    if exchange_name == 'BITSTAMP_BTC_USD':
        from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
        return BitstampBTCUSDExchange
    elif exchange_name == 'BITSTAMP_ETH_EUR':
        from gryphon.lib.exchange.bitstamp_eth_eur import BitstampETHEURExchange
        return BitstampETHEURExchange
    elif exchange_name == 'BITSTAMP_ETH_USD':
        from gryphon.lib.exchange.bitstamp_eth_usd import BitstampETHUSDExchange
        return BitstampETHUSDExchange
    elif exchange_name == 'BITSTAMP_ETH_BTC':
        from gryphon.lib.exchange.bitstamp_eth_btc import BitstampETHBTCExchange
        return BitstampETHBTCExchange
    elif exchange_name == 'BITSTAMP_ETH_EUR':
        from gryphon.lib.exchange.bitstamp_eth_eur import BitstampETHEURExchange
        return BitstampETHEURExchange
    elif exchange_name == 'BITSTAMP_BTC_EUR':
        from gryphon.lib.exchange.bitstamp_btc_eur import BitstampBTCEURExchange
        return BitstampBTCEURExchange
    elif exchange_name == 'BITSTAMP_BCH_BTC':
        from gryphon.lib.exchange.bitstamp_bch_btc import BitstampBCHBTCExchange
        return BitstampBCHBTCExchange
    elif exchange_name == 'BITSTAMP_BCH_USD':
        from gryphon.lib.exchange.bitstamp_bch_usd import BitstampBCHUSDExchange
        return BitstampBCHUSDExchange
    elif exchange_name == 'BITSTAMP_BCH_EUR':
        from gryphon.lib.exchange.bitstamp_bch_eur import BitstampBCHEURExchange
        return BitstampBCHEURExchange
    elif exchange_name == 'KRAKEN_BTC_EUR':
        from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
        return KrakenBTCEURExchange
    elif exchange_name == 'KRAKEN_BTC_USD':
        from gryphon.lib.exchange.kraken_btc_usd import KrakenBTCUSDExchange
        return KrakenBTCUSDExchange
    elif exchange_name == 'KRAKEN_BTC_CAD':
        from gryphon.lib.exchange.kraken_btc_cad import KrakenBTCCADExchange
        return KrakenBTCCADExchange
    elif exchange_name == 'BITFINEX_BTC_USD':
        from gryphon.lib.exchange.bitfinex_btc_usd import BitfinexBTCUSDExchange
        return BitfinexBTCUSDExchange
    elif exchange_name == 'ITBIT_BTC_USD':
        from gryphon.lib.exchange.itbit_btc_usd import ItbitBTCUSDExchange
        return ItbitBTCUSDExchange
    elif exchange_name == 'OKCOIN_BTC_USD':
        from gryphon.lib.exchange.okcoin_btc_usd import OKCoinBTCUSDExchange
        return OKCoinBTCUSDExchange
    elif exchange_name == 'QUADRIGA_BTC_CAD':
        from gryphon.lib.exchange.quadriga_btc_cad import QuadrigaBTCCADExchange
        return QuadrigaBTCCADExchange
    elif exchange_name == 'COINBASE_BTC_USD':
        from gryphon.lib.exchange.coinbase_btc_usd import CoinbaseBTCUSDExchange
        return CoinbaseBTCUSDExchange
    elif exchange_name == 'GEMINI_BTC_USD':
        from gryphon.lib.exchange.gemini_btc_usd import GeminiBTCUSDExchange
        return GeminiBTCUSDExchange
    elif exchange_name == 'GEMINI_ETH_USD':
        from gryphon.lib.exchange.gemini_eth_usd import GeminiETHUSDExchange
        return GeminiETHUSDExchange
    elif exchange_name == 'GEMINI_ETH_BTC':
        from gryphon.lib.exchange.gemini_eth_btc import GeminiETHBTCExchange
        return GeminiETHBTCExchange
    elif exchange_name == 'POLONIEX_ETH_BTC':
        from gryphon.lib.exchange.poloniex_eth_btc import PoloniexETHBTCExchange
        return PoloniexETHBTCExchange
    else:
        raise exceptions.ExchangeNotIntegratedError(exchange_name)




"""
Arbitrage library. Takes in orderbooks and gives back information about arbitrage
opportunities present in them.

The current orderbook format is a dictionary of bid and ask exchange orders. This could
be changed as part of a larger refactor to bring all our orderbook representations to a
single standard.

IMPORTANT:
  - the 'Order' used here is gryphon.lib.exchange.exchange_order, not the database
    order model.
"""

from collections import defaultdict
import itertools

from cdecimal import Decimal

from gryphon.lib.logger import get_logger
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook
from gryphon.lib.money import Money
from gryphon.lib.metrics import midpoint

logger = get_logger(__name__)


CURRENCY_MISMATCH_ERROR_MESSAGE = """\
Orderbooks do not have the same volume currency. A: %s, B: %s\
"""


class MismatchedVolumeCurrenciesError(Exception):
    def __init__(self, cur_a, cur_b):
        self.message = CURRENCY_MISMATCH_ERROR_MESSAGE % (cur_a, cur_b)


class Cross(object):
    """
    Represents a cross between the bids and asks two orderbooks, whether or not that
    cross is a profitable arbitrage opportunity.
    """
    def __init__(self, volume, revenue, fees, buy_ob=None, sell_ob=None, buy_ex=None, sell_ex=None):
        self.volume = volume
        self.revenue = revenue
        self.fees = fees
        self.buy_orderbook = buy_ob
        self.sell_orderbook = sell_ob

        self.buy_exchange = buy_ob['asks'][0].exchange if buy_ob else None
        self.sell_exchange = sell_ob['bids'][0].exchange if sell_ob else None

    @property
    def volume_currency(self):
        return self.buy_exchange.volume_currency

    @property
    def price_currency(self):
        return self.buy_exchange.currency

    @property
    def profit(self):
        return self.revenue - self.fees

    def __nonzero__(self):
        """
        A cross is falsy if there is no overlap volume.
        """
        return bool(self.volume)


def detect_cross(ob1, ob2, ignore_unprofitable=True):
    """
    Look for orderbook overlap between two exchanges that could be arbitraged in either
    direction.

    Returns a Cross or None if no overlap found.
    """
    cross = detect_directional_cross(ob1, ob2, ignore_unprofitable)

    if cross is None:
        cross = detect_directional_cross(ob2, ob1, ignore_unprofitable)

    return cross


def detect_directional_cross(buy_ob, sell_ob, ignore_unprofitable=True):
    """
    Calculates the volume by which buy_ob's asks cut into sell_ob's bids, and the
    profit that could be gleaned if one could take the arbitrage. By default will not
    return a cross if it is not profitable to take advantage of given the fee brackets
    on the two pairs. This can be turned off by setting the argument ignore_unprofitable
    to False.

    Returns a Cross object containing the total volume of the overlap, the expected
    revenue and expected fees if one were to take the full opportunity. By default

    Notes:
      - The orderbook arguments are named for the action you take, not the type of order
        you are looking at. If you want to buy on ob1 and sell on ob2, you look at the
        asks on ob1 (buy_ob) and the bids on ob2 (sell_ob).
      - This supports considering orderbooks with different price currencies, since
        that is a common operation. However, if you wish to use this for price
        currencies with high hourly or even minute-ly volatility (like BTC, ETH or any
        other cryptocurrency), the speed at which exchange rates are updated in the
        builtin exchange rate service--OpenExchangeRates--may not be fast enough. It's
        strongly recommended that users do their own research on this.
      - It's important that the market order fees for the exchanges you use have be
        accurately configured.
      - In this usage 'volume' refers to the area of the overlap, which is one-half the
        total volume that would be required in orders to take the opportunity, since
        there are two exchanges.
    """

    if not buy_ob['asks'] or not sell_ob['bids']:  # A degenerate orderbook.
        return None

    buy_ex = buy_ob['asks'][0].exchange
    sell_ex = sell_ob['bids'][0].exchange

    # Detect the volume currency and check for a mismatch.
    if buy_ex.volume_currency != sell_ex.volume_currency:
      raise MismatchedVolumeCurrenciesError(
          buy_ex.volume_currency,
          sell_ex.volume_currency,
      )

    volume_currency = buy_ex.volume_currency

    # We use the buy exchange's price currency as our ground.
    base_price_currency = buy_ex.currency

    # Initialize the variables we use in the iteration phase.
    total_volume = Money('0', volume_currency)
    total_revenue = Money('0', base_price_currency)
    total_fees = Money('0', base_price_currency)

    ask_index = -1
    bid_index = -1
    ask_remaining_volume = Money('0', volume_currency)
    bid_remaining_volume = Money('0', volume_currency)
    ask = None
    bid = None

    while ask_index < len(buy_ob['asks']) - 1 and bid_index < len(sell_ob['asks']) - 1:
        if ask_remaining_volume == Money('0', volume_currency):
            ask_index += 1
            ask = buy_ob['asks'][ask_index]
            ask_price = ask.price.to(base_price_currency)
            ask_remaining_volume = ask.volume

        if bid_remaining_volume == Money('0', volume_currency):
            bid_index += 1
            bid = sell_ob['bids'][bid_index]
            bid_price = bid.price.to(base_price_currency)
            bid_remaining_volume = bid.volume

        if bid_price > ask_price:  # Found a cross
            margin = bid_price - ask_price
            volume = None

            if bid_remaining_volume > ask_remaining_volume:
                # This bid eats the whole ask.
                volume = ask_remaining_volume
            else:
                # This bid only eats part of the ask.
                volume = bid_remaining_volume

            revenue = margin * volume.amount
            total_revenue += revenue

            total_fees += bid_price * volume.amount * bid.exchange.market_order_fee
            total_fees += ask_price * volume.amount * ask.exchange.market_order_fee

            total_volume += volume
            bid_remaining_volume -= volume
            ask_remaining_volume -= volume
        else:
            break

    if total_volume:
        cross = Cross(total_volume, total_revenue, total_fees, buy_ob, sell_ob)

        if ignore_unprofitable is False:
            return cross
        elif total_revenue > total_fees:
            return cross
        else:
            return None
    else:
        return None


def get_executable_volume(cross, buy_ex_balance, sell_ex_balance):
    """
    Given a cross between two exchanges and balance information for accounts on both
    exchanges, determine how much volume of the opportunity could be taken by the
    trader.
    """

    if not cross:
        return None

    # Sell max is just whichever is lower, the volume available or the balance we have.
    sell_max = min(cross.volume, sell_ex_balance[cross.volume_currency])

    # It's a little more complicated on the buy side.
    buy_max = max_buy_volume(buy_ex_balance[cross.price_currency], cross.buy_orderbook)

    return min(buy_max, sell_max)


def detect_crosses_between_many_orderbooks(orderbooks, ignore_unprofitable=True):
    """
    Takes in a list of orderbooks and returns a list of crosses between those
    orderbooks sorted by profitability.
    """

    crosses = []

    for pair in itertools.combinations(orderbooks, 2):
        cross = detect_cross(pair[0], pair[1], ignore_unprofitable)

        if cross is not None:
            crosses.append(cross)

    crosses = sorted(crosses, key=lambda c: c.profit, reverse=True)

    return crosses


def max_buy_volume(balance, buy_orderbook):
    """
    What is the maximum volume we can buy on the given orderbook with the given balance?
    This is more complicated than it initially appears due to slippage and fees.

    It feels like this belongs in it's own library, but there isn't a clear place for it
    I can think of, so it remains here for now, nearby it's main usage.
    """
    buy_ex = buy_orderbook['asks'][0].exchange
    fee = buy_ex.market_order_fee

    vol_currency = buy_ex.volume_currency

    balance_remaining = balance
    volume_available = Money('0', vol_currency)

    for order in buy_orderbook['asks']:
        total_order_price = order.price * order.volume.amount
        total_order_fee = total_order_price * fee

        total_order_cost = total_order_price + total_order_fee

        if total_order_cost <= balance_remaining:
            balance_remaining -= total_order_cost
            volume_available += order.volume
        else:
            # With our given balance we want to use all of it, so we have:
            # (1 + fee) * (volume * price) = balance
            # volume = (balance / (1 + fee)) / price
            last_volume = (
                (balance_remaining.amount / (Decimal('1') + fee)) / order.price.amount
            )

            last_volume = Money(last_volume, vol_currency)

            volume_available += last_volume

            break

    return volume_available




import json

from autobahn.twisted.websocket import connectWS, WebSocketClientFactory
from cdecimal import Decimal
from twisted.internet import defer, ssl

import gryphon.data_service.consts as consts
import gryphon.data_service.util as util
from gryphon.data_service.websocket_client import EmeraldWebSocketClientProtocol
from gryphon.data_service.pollers.orderbook.websocket.websocket_orderbook_poller import WebsocketOrderbookPoller
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class BitfinexOrderbookWebsocket(EmeraldWebSocketClientProtocol, WebsocketOrderbookPoller):
    def __init__(self):
        self.exchange_name = u'BITFINEX_BTC_USD'
        self.ping_interval = 5

    def connect_to_websocket(self):
        logger.info('Connecting to websocket')

        factory = WebSocketClientFactory('wss://api2.bitfinex.com:3000/ws')

        factory.protocol = type(self)

        if factory.isSecure:
            context_factor = ssl.ClientContextFactory()
        else:
            context_factor = None

        connectWS(factory, context_factor)

    def subscribe_to_websocket(self):
        logger.info('Subscribing to websocket')

        data = {
            'Event': 'subscribe',
            'Channel': 'book',
            'Pair': 'BTCUSD',
            'Prec': 'P0',
            'Len': 'FULL',  # Undocumented guess, lol.
        }

        self.sendMessage(json.dumps(data))

    # WEBSOCKET CLIENT FUNCTIONS

    @defer.inlineCallbacks
    def onOpen(self):
        logger.info('Connected to websocket')

        self.redis = yield util.setup_redis()
        binding_key = '%s.orderbook.tinker' % self.exchange_name.lower()
        self.producer = yield util.setup_producer(consts.ORDERBOOK_QUEUE, binding_key)

        # Reset everything.
        self.orderbook = {'bids': None, 'asks': None}
        self.bids_dict = {}
        self.asks_dict = {}
        self.channel_id = None
        self.have_gotten_base_orderbook = False
        self.last_amqp_push = 0

        yield self.redis.set(self.orderbook_key, None)

        self.subscribe_to_websocket()

    @defer.inlineCallbacks
    def onMessage(self, payload, isBinary):
        should_continue = yield self.check_should_continue()

        if not should_continue:
            yield self.redis.set(self.orderbook_key, None)
            return

        payload = json.loads(payload, parse_float=Decimal)

        # Initial subscription confirmation.
        if 'Event' in payload and payload['Event'] == 'subscribed':
            self.channel_id = payload['ChanId']
            return

        channel_id = payload[0]

        assert(channel_id == self.channel_id)

        # The first message after subscribing is always the base orderbook.
        if not self.have_gotten_base_orderbook:
            base_orderbook = payload[1]
            self.set_base_orderbook(base_orderbook)
            self.have_gotten_base_orderbook = True
            self.publish_orderbook()

            return

        # If we are not in one of the two initial special cases
        # then we have an orderbook change event of the format:
        # [channel_id, price, num_orders, volume_diff]
        change = payload[1:]

        self.apply_change_to_orderbook(change)
        self.publish_orderbook()

    def set_base_orderbook(self, base_orderbook):
        for order in base_orderbook:
            self.apply_change_to_orderbook(order)

    def apply_change_to_orderbook(self, change):
        # change is [0, 9, 20935.8851]
        # price, num_orders, volume
        # volume is + for bids, - for asks
        price = change[0]
        volume = change[2]

        if volume > 0:
            self.bids_dict[price] = volume
        elif volume < 0:
            self.asks_dict[price] = abs(volume)
        else:
            # When volume is 0 we don't know if this is for bids or asks we can safely
            # remove both, since there will never be a bid and an ask at the same price.
            self.bids_dict[price] = 0
            self.asks_dict[price] = 0

    def get_orderbook_to_publish(self):
        # We store the orderbook internally as a bid and ask dict, so we convert it here
        # to sorted lists of [price, volume] orders.
        bids = []
        asks = []

        for price, volume in sorted(self.bids_dict.iteritems(), reverse=True):
            if volume > 0:
                bids.append([str(price), str(volume), ''])

        for price, volume in sorted(self.asks_dict.iteritems()):
            if volume > 0:
                asks.append([str(price), str(volume), ''])

        orderbook = {}
        orderbook['bids'] = bids
        orderbook['asks'] = asks

        return orderbook

    def print_orderbook(self):
        """Print current orderbook in format similar to Bitfinex's homepage orderbook"""
        bids = self.orderbook['bids']
        asks = self.orderbook['asks']

        if not bids or not asks:
            return

        # Pushes orderbook to bottom of terminal for a live-updating effect.
        logger.info("\n" * 100)
        logger.info('{:=^78}'.format(' ORDER BOOK '))
        logger.info('{: ^39}{: ^39}'.format('BIDS', 'ASKS'))

        titles = "{:^7} | {:^12} | {:^12} ||| {:^12} | {:^12} | {:^7}".format(
            'Price',
            'Amount',
            'Cumulative',
            'Cumulative',
            'Amount',
            'Price',
        )

        logger.info(titles)

        bid_sum = 0
        ask_sum = 0

        while bids and asks:
            bid = bids.pop(0)
            ask = asks.pop(0)

            bid_sum += bid[1]
            ask_sum += ask[1]

            line = "{:^7.1f} | {:^12.1f} | {:^12.1f} ||| {:^12.1f} | {:^12.1f} | {:^7.1f}".format(
                bid[0],
                bid[1],
                bid_sum,
                ask_sum,
                ask[1],
                ask[0]
            )

            logger.info(line)



from twisted.python import log

from gryphon.data_service.pollers.volume.volume_poller import VolumePoller


class CoinbaseCADVolume(VolumePoller):
    def __init__(self):
        self.exchange_name = u'COINBASE_BTC_CAD'
        self.url = 'https://api.exchange.coinbase.com/products/BTC-CAD/stats'

    def parse_volume_json(self, volume_dict):
        """
        Parses this:
        {
            "open":"377.99000000",
            "high":"382.25000000",
            "low":"351.16000000",
            "volume":"11457.39541978"
        }
        """

        try:
            return volume_dict['volume']
        except Exception as e:
            log.msg('Error getting Coinbase CAD Volume %s' % e)



"""
This is a super simple market making strategy to demonstrate use of the Gryphon
framework. For being only 25 lines of code, this strategy has some very subtle logic to
it.

Every tick, we place orders on both sides of the orderbook at a 1% spread back from the
midpoint on the Bitstamp BTCUSD pair. The orders start at a size of 0.005 BTC. If either
of these orders are filled in any volume, the strategy is now at a net position. We want
to limit our absolute risk, so the strategy has a maximum position it will enter. To
avoid going past our max position, the order size on the side of our position is
decreased so that we should never be offering an order that would take us past our
maximum if it were filled.

This strategy is just an example, and should not be used for real trading.
"""

from cdecimal import Decimal

from gryphon.execution.strategies.base import Strategy
from gryphon.lib import market_making as mm
from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts


class SuperSimpleMarketMaking(Strategy):
    def tick(self, current_orders):
        self.harness.bitstamp_btc_usd.cancel_all_open_orders()

        ob = self.harness.bitstamp_btc_usd.get_orderbook()

        bid_price, ask_price = mm.midpoint_centered_fixed_spread(ob, Decimal('0.01'))

        bid_volume, ask_volume = mm.simple_position_responsive_sizing(
            Money('0.005', 'BTC'),
            self.position,
        )

        if bid_volume > 0:
            self.harness.bitstamp_btc_usd.limit_order(Consts.BID, bid_volume, bid_price)

        if ask_volume > 0:
            self.harness.bitstamp_btc_usd.limit_order(Consts.ASK, ask_volume, ask_price)




from delorean import Delorean

from gryphon.data_service.auditors.orderbook_auditor import OrderbookAuditor


class CoinbaseOrderbookAuditor(OrderbookAuditor):
    def __init__(self):
        super(CoinbaseOrderbookAuditor, self).__init__()
        self.exchange_name = 'COINBASE_BTC_USD'
        product_id = 'BTC-USD'
        self.orderbook_url = 'https://api.pro.coinbase.com/products/%s/book?level=3' % product_id
        self.audit_time = 60

        self.acceptable_changes_threshold = 40

    def get_timestamp(self, new_orderbook):
        return Delorean().datetime



from __future__ import absolute_import

import gryphon.lib; gryphon.lib.monkeypatch_decimal_to_cdecimal()

from gryphon.lib.forex import USDCurrencyConverter

import json
import decimal
# need absolute_import from above so that this doesn't load our current file
import money as super_money


class Money(super_money.Money):
    FIAT_CURRENCIES = ["USD", "CAD", "EUR"]
    CRYPTO_CURRENCIES = ["BTC", "ETH", "BCH"]
    CURRENCIES = FIAT_CURRENCIES + CRYPTO_CURRENCIES

    def __init__(self, amount="0", currency=None):
        if isinstance(amount, basestring):
            amount = amount.replace(",", "")

        try:
            self.amount = decimal.Decimal(amount)
        except decimal.InvalidOperation:
            raise ValueError(
                "amount value could not be converted to Decimal(): '{}'".format(amount),
            )

        if currency not in self.CURRENCIES:
            raise ValueError("invalid currency value: '{}'".format(currency))

        self.currency = currency

    def round_to_decimal_places(self, places, rounding=decimal.ROUND_UP):
        """
        Round a money object to n decimal places.

        Useful for truncating BTC amounts before sending them to exchange APIs
        Default rounds up (??? not sure why)
        but you can pass in your own rounding setting
        """
        bucket = decimal.Decimal(10) ** -places
        return self.round_to_bucket(bucket, rounding=rounding)

    def round_to_bucket(self, bucket, rounding=decimal.ROUND_DOWN):
        """
        Round a money object to the nearest bucket.

        Useful for getting friendly wire/btc amounts (eg: 1,223.45 => 1,000.00)
        Default rounds down, but you can pass in your own rounding setting
        """
        rounded_amount = (self.amount / bucket).to_integral_value(rounding=rounding) * bucket
        return self.__class__(rounded_amount, self.currency)

    def __repr__(self):
        amount = self.amount
        # fix case of 0E-8
        if amount == 0:
            amount = 0
        return "{} {}".format(self.currency, amount)

    def __unicode__(self):
        if self.currency == 'BTC':
            return u"{} {:,.8f}".format(self.currency, self.amount)
        else:
            return u"{} {:,.2f}".format(self.currency, self.amount)

    def to_json(self):
        return repr(self)

    def to(self, currency, date=None, exchange_rate_to_usd=None):
        """Return equivalent money object in another currency from a specific date"""
        if currency == self.currency:
            return self

        if exchange_rate_to_usd:
            if not isinstance(exchange_rate_to_usd, decimal.Decimal):
                raise ValueError("exchange_rate_to_usd must be a Decimal")
            if exchange_rate_to_usd <= 0:
                raise ValueError("exchange_rate_to_usd must be a positive number")
            if currency != "USD":
                raise ValueError("Must be converting to USD to use exchange_rate_to_usd")
            usd_amount = self.amount * exchange_rate_to_usd
            return self.__class__(usd_amount, "USD")

        if date:
            a = USDCurrencyConverter.historical_rate(self.currency, date)
            b = USDCurrencyConverter.historical_rate(currency, date)
        else:
            a = USDCurrencyConverter.rate(self.currency)
            b = USDCurrencyConverter.rate(currency)
        rate = b / a
        amount = self.amount * rate
        return self.__class__(amount, currency)



import pyximport; pyximport.install()

from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.tests.logic.exchange_wrappers.public_methods import ExchangePublicMethodsTests


class TestBitstampPublicMethods(ExchangePublicMethodsTests):
    def setUp(self):
        self.exchange = BitstampBTCUSDExchange()




"""
Just a few exercises for our configuration library.
"""
import pyximport; pyximport.install()
import unittest

from cdecimal import Decimal

from gryphon.lib.money import Money
from gryphon.execution.strategies.builtin.improved_market_making import ImprovedMarketMaking


class TestSimpleStrategy(unittest.TestCase):
    def test_empty_config(self):
        configuration = {
            'platform': {},
            'strategy': {},
            'exchanges': {},
        }
      
        strat = ImprovedMarketMaking(None, None, configuration['strategy'])

        assert strat.spread == Decimal('0.10')
        assert strat.base_volume == Money('0.005', 'BTC')

    def test_simple(self):
        configuration = {
            'platform': {},
            'strategy': {
                'base_volume': Money('1', 'BTC'),
                'spread': Decimal('1.0'),
            },
            'exchanges': {},
        }
      
        strat = ImprovedMarketMaking(None, None, configuration['strategy'])

        assert strat.spread == Decimal('1')
        assert strat.base_volume == Money('1', 'BTC')



# -*- coding: utf-8 -*-
import logging
import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler

logger = logging.getLogger(__name__)


class LogoutHandler(AdminBaseHandler):
    @tornado.web.authenticated
    def post(self):
        self.logout_user()
        self.redirect('login')




"""
Our second run at a fundamental value claculator, a bit smarter.
"""

from gryphon.lib.exchange.retry import exchange_retry
from gryphon.lib.money import Money
from gryphon.execution.lib import conf


@exchange_retry()
def calculate(algo, fundamental_value_balance_map):
    """
    Weighted fundamental value including only exchanges that can both buy and sell.
    """
    # TODO - store the buffer value in a config or some other thoughtout way.
    buffer_value = algo.config['volume_currency_buffer_value']

    weighted_fundamental_value = 0
    weight_sum = 0

    participating_exchanges = []

    for exchange_name, exchange in fundamental_value_balance_map.iteritems():
        # If you have bitcoin and are below your maximum fiat, you can sell.
        can_sell = (
            exchange[algo.volume_currency.lower()] > buffer_value and
            exchange['fiat'] < exchange['maximum'] - buffer_value.to('USD') and
            'bid_quote' in exchange and
            exchange['bid_quote']
        )

        # If you have enough fiat you can buy.
        can_buy = (
            exchange['fiat'] > buffer_value.to('USD') and
            'ask_quote' in exchange and
            exchange['ask_quote']
        )

        if can_sell and can_buy:
            participating_exchanges.append(exchange_name)
            weight_sum += conf.fv_v2_weights[algo.volume_currency][exchange_name]

    if not participating_exchanges:
        raise Exception('No exchanges to calculate fundamental value')

    trusted_exchanges = [
        e.strip().upper() for e in algo.config['trusted_exchanges'].split(',')
    ]

    if not any([e in trusted_exchanges for e in participating_exchanges]):
        raise Exception('No trusted exchanges in participating_exchanges')

    for exchange_name in participating_exchanges:
        exchange = fundamental_value_balance_map[exchange_name]
        exchange_fv = (exchange['ask_quote'] + exchange['bid_quote']) / 2
        exchange_weight = conf.fv_v2_weights[algo.volume_currency][exchange_name]
        normalized_weight = exchange_weight / weight_sum

        weighted_fundamental_value += exchange_fv * normalized_weight

    algo.log('Participating Exchanges: %s', participating_exchanges, 'yellow')

    return weighted_fundamental_value, participating_exchanges




# -*- coding: utf-8 -*-
import json

import termcolor as tc
from twisted.internet import defer
from twisted.internet.task import LoopingCall
from twisted.python import log

import gryphon.data_service.consts as consts
from gryphon.data_service.pollers.request_poller import RequestPoller
import gryphon.data_service.util as util


class TradesPoller(RequestPoller):
    @property
    def trade_id_key(self):
        return '%s_most_recent_trade_id' % self.exchange_name.lower()

    @property
    def heartbeat_key(self):
        return '%s_trades_heartbeat' % self.exchange_name.lower()

    @defer.inlineCallbacks
    def start(self):
        self.redis = yield util.setup_redis()

        binding_key = '%s.trades.tinker' % self.exchange_name.lower()
        self.producer = yield util.setup_producer(consts.TRADES_QUEUE, binding_key)

        raw_most_recent_trade_id = yield self.redis.get(self.trade_id_key)

        if raw_most_recent_trade_id:
            self.most_recent_trade_id = int(raw_most_recent_trade_id)
        else:
            self.most_recent_trade_id = 0

        # We are not using RequestPoller's start() function because we want to set
        # now=False. This is a bit of a hack to give the producer time to get set up
        # before we start publishing messages.
        self.looping_call = LoopingCall(self.get_request)
        self.looping_call.start(self.poll_time, now=False)

    @defer.inlineCallbacks
    def parse_response(self, resp_obj):
        trades = self.parse_trades(resp_obj)

        # oldest trade is the first one in the list
        trades = sorted(trades, key=lambda k: k['trade_id'])

        for trade in trades:
            if trade['trade_id'] > self.most_recent_trade_id:
                trade['price_currency'] = trade['price'].currency
                trade['price'] = unicode(trade['price'].amount)
                trade['volume_currency'] = trade['volume'].currency
                trade['volume'] = unicode(trade['volume'].amount)
                trade['timestamp'] = int(trade['timestamp'])
                trade_string = json.dumps(trade, ensure_ascii=False)
                self.producer.publish_message(trade_string)
                self.most_recent_trade_id = trade['trade_id']

        if (trades[0]['trade_id'] > self.most_recent_trade_id
                and self.most_recent_trade_id > 0):
            log.msg(tc.colored('Trades Missed on %s' % self.exchange_name, 'red'))

        yield self.redis.set(self.trade_id_key, self.most_recent_trade_id)

    @defer.inlineCallbacks
    def parse_trades(self, trade_dict):
        pass



from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class CoinbaseBTCUSDOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'COINBASE_BTC_USD'
        self.product_id = 'BTC-USD'
        self.base_url = 'https://api.pro.coinbase.com/products/%s/book?level=3'
        self.url = self.base_url % self.product_id
        self.poll_time = 2

    def parse_order(self, order):
        return [order[0], order[1], '']




"""
`gryphon-execute strategy` takes configration parameters both from the commannd line and
from a .conf file. This library is concerned with standardizing the format of the
config dictionary that is passed into the harness, and handling the rules for combining
configuration settings from both sources.

Just a few functions that make our configuration handling cleaner.

The rule is: 
  1) From any source, if something isn't specified explicitly, it is
      left out of the dictionary, or put as None, and any other value from any
      source will override it.
  2) Always specify defaults in code.
  3) At no point is there a any guarantee a setting is in the configuration object.
"""

import ConfigParser
import argparse

from cdecimal import Decimal, InvalidOperation

from gryphon.lib import configuration
from gryphon.lib.money import Money
from gryphon.lib.exchange import exchange_factory


def get_command_line_configuration(known_args, extra_args):
    command_line_configuration = {
        'platform': {
            'strategy': known_args.strategy,
            'config_file': known_args.config_file,
            'builtin': known_args.builtin,
            'heartbeat': known_args.heartbeat,
            'sentry': known_args.sentry,
            'emerald': known_args.emerald,
            'audit': known_args.audit,
            'execute': known_args.execute,
        },
        'strategy': {
            'tick_sleep': configuration.parse_configurable_value(known_args.tick_sleep),
        },
    }

    extra_strategy_configuration = parse_extra_strategy_args(extra_args)

    command_line_configuration['strategy'].update(extra_strategy_configuration)

    return command_line_configuration


def parse_extra_strategy_args(args):
    """
    Turns a list of unknown arguments from the controller into a dictionary. This is
    necessary because the framework doesn't know what argument a strategy might
    require.

    This ONLY supports named arguments, and only supports the syntax '--name name', not
    --name=name.

    There are likely more graceful ways to do this, such as having Strategies know which
    of their properties are configurable and allowing them to register extra arguments,
    but this will do for now.

    TODO:
      - don't print messages from argparse here, just throw an exception or something.
      - the hackery here to support boolean flags (the nargs/const condition) means that
        if you don't give an value for an argument that should have one, no error is
        thrown, and that value is set to 'True'. In most cases this should be caught
        further down the line, but we really should handle it here.
    """

    settings = [x for x in args if x[:2] == '--']
    
    parser = argparse.ArgumentParser()

    for setting_name in settings:
        parser.add_argument(setting_name, action='store', nargs='?', const=True)

    output = vars(parser.parse_args(args))  # Vars turns a Namespace into a dict.

    output = {k: configuration.parse_configurable_value(v) for k, v in output.items()}

    return output


def get_conf_file_configuration(conf_filename, strategy_name):
    """
    If there was a config file specified, load it. If not, look for [strategy_name].conf
    in the current directory. If neither is found just return an initialized 
    configuration object that is otherwise empty.
    """

    if conf_filename is None:
        if strategy_name[-4:] == '.pyx':
            conf_filename = '%s.conf' % strategy_name[:-4]
        else:
            conf_filename = '%s.conf' % strategy_name

    file_configuration = configuration.read_config_from_file(conf_filename)

    standardized_configuration = format_file_config_to_standard(file_configuration)

    return standardized_configuration


def format_file_config_to_standard(file_configuration):
    """
    Regardless of what is in the .conf file, we need to output a dictionary with the
    keys platform, strategy, and exchanges. We also need to correctly categorize any
    exchange configuration settings in the .conf file--specified by a section with the
    same name as an exchange trading pair--under the 'exchanges' key.
    """
    new_configuration = {'platform': {}, 'strategy': {}, 'exchanges': {}}

    for section_name, values in file_configuration.items():
        if section_name in exchange_factory.ALL_EXCHANGE_KEYS:
            new_configuration['exchanges'][section_name] = values
        else:
            new_configuration[section_name] = values

    return new_configuration


def combine_file_and_command_line_config(file_config, command_line_config):
    """
    We have our configuration from the .conf, and we have the configuration from the
    command line. If there are any settings specified in both, take the one from the
    command line.
    """

    final_configuration = file_config.copy()

    configuration.dict_update_override(
        final_configuration['platform'],
        command_line_config['platform'],
    )

    configuration.dict_update_override(
        final_configuration['strategy'],
        command_line_config['strategy'],
    )

    return final_configuration




import pyximport; pyximport.install()

import os
import unittest
import mock
from delorean import Delorean

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money

from algos.naive import Naive
from algos.harness import Harness
from gryphon.lib.models.order import Order
from gryphon.lib.models.exchange import Position, Balance
from brain import Brain

import logging
logger = logging.getLogger(__name__)

class TestHarness():

    def setUp(self):
        self.db = mock.Mock()

        self.algo = mock.Mock()
        self.algo.brain = Brain()

        exchange_data = mock.Mock()
        exchange_data.position = Position()
        exchange_data.balance = Balance()
        self.algo.exchange_data = exchange_data

        exchange = mock.Mock()
        exchange.name = "TestExchange"
        exchange.currency = "USD"
        exchange.get_order_book_resp = mock.Mock(return_value=None)
        exchange.exchange_data = mock.Mock(return_value=exchange_data)
        self.algo.exchange = exchange

        self.algo.params = {'fundamental_depth':Money(1, 'BTC')}

        self.algo.fundamental_exchanges = []

        # We have a 1 BTC bid for 500 on the book
        self.order = Order("Test", Consts.BID, Money("1", 'BTC'), Money("500", 'USD'), self.algo.exchange, '123')
        self.algo._get_open_orders = mock.Mock(return_value=[self.order])

        self.algo._get_orders_by_order_ids = mock.Mock(return_value=[self.order])

        self.harness = Harness(self.algo, self.db)

    def test_eaten_orders(self):
        # Our open order has been taken
        self.algo.exchange.open_orders = mock.Mock(return_value=[])
        self.algo.calc_fundamental_value = mock.Mock(return_value=Money('500', 'USD'))

        open_orders = self.harness.fetch_tick_data(None)
        eaten_order_ids, current_orders = self.harness.get_current_orders(open_orders)
        eaten_order_ids.should.have.length_of(1)
        eaten_order_ids.should.contain('123')
        current_orders.should.be.empty

    def test_current_orders(self):
        # our open order is still there
        self.algo.exchange.open_orders = mock.Mock(return_value=[{'id': '123'}])
        self.algo.calc_fundamental_value = mock.Mock(return_value=Money('500', 'USD'))

        open_orders = self.harness.fetch_tick_data(None)
        eaten_order_ids, current_orders = self.harness.get_current_orders(open_orders)
        eaten_order_ids.should.be.empty
        current_orders.should.have.length_of(1)
        current_orders[0].should.have.key('id').being.equal('123')

    def test_accounting(self):
        v = Money("1", 'BTC')
        p = Money("500", 'USD')

        # our open order was completely taken
        multi_order_details = {'123': {'btc_total': v,
          'trades': [{'btc': v,
            'trade_id': None,
            'fiat': p,
            'fee': Money('0', 'USD'),
            'time': Delorean().epoch,
            }],
          'type': Consts.BID,
          'fiat_total': p
          }}
        self.algo.exchange.multi_order_details = mock.Mock(return_value=multi_order_details)
        self.algo.fundamental_value = Money("500", 'USD')
        self.harness.run_accounting(['123'], [])
        self.algo.exchange_data.position['USD'].should.equal(Money("0", 'USD') - p)
        self.algo.exchange_data.position['BTC'].should.equal(v)

        self.order.status.should.equal("FILLED")

        self.order.trades.should.have.length_of(1)
        self.order.trades[0].volume.should.equal(v)
        self.order.trades[0].price.should.equal(p)

    def test_accounting_cancelled(self):
        v = Money("0", 'BTC')
        p = Money("0", 'USD')

        # our open order was cancelled
        multi_order_details = {'123': {'btc_total': v,
          'trades': [],
          'type': Consts.BID,
          'fiat_total': p,
          }}
        self.algo.exchange.multi_order_details = mock.Mock(return_value=multi_order_details)

        self.algo.fundamental_value = Money("500", 'USD')
        self.harness.run_accounting(['123'], [])
        self.algo.exchange_data.position['USD'].should.equal(Money("0", 'USD') - p)
        self.algo.exchange_data.position['BTC'].should.equal(v)

        self.order.status.should.equal("CANCELLED")

        self.order.trades.should.be.empty

    def test_accounting_partially_filled(self):
        v = Money("0.1", 'BTC')
        p = Money("50", 'USD')

        # our open order was partially
        order_details = {'123':{'btc_total': v,
          'trades': [{'btc': v,
            'trade_id': None,
            'fiat': p,
            'fee': Money('0', 'USD'),
            'time': Delorean().epoch,
            }],
          'type': Consts.BID,
          'fiat_total': p,
          }
         }
        self.algo.exchange.multi_order_details = mock.Mock(return_value=order_details)

        current_orders = [{'id': '123',
          'mode': Consts.BID,
          'price': Money("500", 'USD'),
          'volume_remaining': Money("0.9", 'BTC')}]

        self.algo.fundamental_value = Money("500", 'USD')
        self.harness.run_accounting([], current_orders)
        self.algo.exchange_data.position['USD'].should.equal(Money("0", 'USD') - p)
        self.algo.exchange_data.position['BTC'].should.equal(v)

        self.order.status.should.equal('OPEN')

        self.order.trades.should.have.length_of(1)
        self.order.trades[0].volume.should.equal(v)
        self.order.trades[0].price.should.equal(p)

    def test_accounting_partially_filled_then_cancelled(self):
        v = Money("0.1", 'BTC')
        p = Money("50", 'USD')

        # our open order was partially
        order_details = {'btc_total': v,
          'trades': [{'btc': v,
            'trade_id': None,
            'fiat': p,
            'fee': Money('0', 'USD'),
            'time': Delorean().epoch,
            }],
          'type': Consts.BID,
          'fiat_total': p,
          }
        multi_order_details = {'123': order_details}
        self.algo.exchange.order_details = mock.Mock(return_value=order_details)
        self.algo.exchange.multi_order_details = mock.Mock(return_value=multi_order_details)


        current_orders = [{'id': '123',
          'mode': Consts.BID,
          'price': Money("500", 'USD'),
          'volume_remaining': Money("0.9", 'BTC')}]
        self.algo.fundamental_value = Money("500", 'USD')
        self.harness.run_accounting([], current_orders)
        self.algo.exchange_data.position['USD'].should.equal(Money("0", 'USD') - p)
        self.algo.exchange_data.position['BTC'].should.equal(v)

        # order got cancelled
        self.harness.run_accounting(['123'], [])

        # position should not change
        self.algo.exchange_data.position['USD'].should.equal(Money("0", 'USD') - p)
        self.algo.exchange_data.position['BTC'].should.equal(v)

        # trades should not change
        self.order.trades.should.have.length_of(1)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



"""
Just a quick mixing for parsing a list of exchanges out of the querystring.
"""

DEFAULT_EXCHANGES = ['BITSTAMP', 'ITBIT', 'GEMINI']


class ActiveExchangesMixin():
    def get_active_exchanges(self, custom_defaults=None):
        raw_active_exchanges = self.get_argument('exchanges', None)
        active_exchanges = self.parse_active_exchanges(
            raw_active_exchanges,
            custom_defaults,
        )

        return active_exchanges

    def parse_active_exchanges(self, query_param, custom_defaults=None):
        if query_param is None:
            if custom_defaults is not None:
                return custom_defaults
            else:
                return DEFAULT_EXCHANGES
        else:
            exchange_names = query_param.split(',')
            exchange_names = [e.upper() for e in exchange_names]

            return exchange_names



from collections import defaultdict
import json
import logging

from delorean import Delorean
import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.active_exchanges import ActiveExchangesMixin
from gryphon.dashboards.handlers.orderbooks import defaults
from gryphon.lib import configuration as config_lib
from gryphon.lib.configurable_object import ConfigurableObject
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = logging.getLogger(__name__)


class OrderbooksHandler(AdminBaseHandler, ActiveExchangesMixin, ConfigurableObject):
    def __init__(self, *args, **kwargs):
        super(OrderbooksHandler, self).__init__(*args, **kwargs)

        # Configurable properties with defaults.
        self.price_currency = None
        self.volume_currency = None
        self.infinitisimal = None
        self.infinity = None
        self.zero = 0
        self.trading_pairs = []
        self.DEFAULT_WIDTH = None
        self.pair_name = None

    def configure(self, configuration):
        self.init_configurable('price_currency', configuration)
        self.init_configurable('volume_currency', configuration)
        self.init_configurable('infinitisimal', configuration)
        self.init_configurable('infinity', configuration)
        self.init_configurable('zero', configuration)
        self.init_configurable('trading_pairs', configuration)
        self.init_configurable('DEFAULT_WIDTH', configuration)
        self.init_configurable('pair_name', configuration)

        # Parse as list and turn exchange names into exchange connections.
        self.trading_pair_names = config_lib.parse_configurable_as_list(
            self.trading_pairs,
        )

        self.trading_pair_names = [t.upper() for t in self.trading_pair_names]

        self.trading_pairs = []

        for trading_pair_name in self.trading_pair_names:
            e = exchange_factory.make_exchange_from_key(trading_pair_name)
            self.trading_pairs.append(e)

    @tornado.web.authenticated
    def get(self, collection_name):
        configuration = None

        if (collection_name in self.configuration['orderbook_collections']
                and collection_name in defaults.BUILTIN_ORDERBOOK_CONFIGS.keys()):

            # Combine the two configurations intelligently.
            configuration = defaults.BUILTIN_ORDERBOOK_CONFIGS[collection_name].copy()

            config_lib.dict_update_override(
                configuration,
                self.configuration['orderbook_collections'][collection_name],
            )
        elif collection_name in self.configuration['orderbook_collections']:
            configuration = self.configuration['orderbook_collections'][collection_name]
        elif collection_name in defaults.BUILTIN_ORDERBOOK_CONFIGS.keys():
            configuration = defaults.BUILTIN_ORDERBOOK_CONFIGS[collection_name]
        else:
            self.redirect('/404')
            return

        self.configure(configuration)

        template_args = self.generate_data()

        if self.get_argument('json', False) is not False:
            self.write(json.dumps({
                'levels': template_args['levels_for_exchange'],
                'orders': template_args['open_orders_for_exchange'],
            }))
        else:
            self.render_template('orderbook.html', args=template_args)

    def generate_data(self):
        gds_active = self.is_gds_connection_active()

        if gds_active is False:
            return self.generate_null_data()

        # Get query string parameters.
        query_string_high = self.get_argument('high', None)
        query_string_low = self.get_argument('low', None)
        show_orders = self.get_argument('show_orders', False) != False
        at_time = self.get_argument('at_time', None)
        no_lookback_limit = self.get_argument('no_lookback_limit', None)

        selected_exchanges = self.get_active_exchanges(self.trading_pair_names)

        # Some simple formatting of our query string parameters.
        self.no_lookback_limit = True if no_lookback_limit else False

        if at_time:
            at_time = parse(at_time).datetime

        # Get the data.
        open_orders_for_exchange = {}

        open_orders_for_exchange = self.get_open_orders_by_exchange(at_time)

        levels_for_exchange, low_bid, high_ask = self.get_orderbook_levels(
            self.gds_db,
            self.trading_pairs,
            at_time,
            selected_exchanges,
        )

        levels_for_exchange = self.format_levels_for_graphing(levels_for_exchange)

        high, low = self.determine_graph_default_range(
            query_string_high,
            query_string_low,
            low_bid,
            high_ask,
            open_orders_for_exchange,
        )

        args = {
            'gds_active': gds_active,
            'levels_for_exchange': levels_for_exchange,
            'selected_exchanges': selected_exchanges,
            'open_orders_for_exchange': open_orders_for_exchange,
            'high': high,
            'low': low,
            'pair_name': self.pair_name,
            'show_orders': show_orders,
            'at_time': None if not at_time else at_time.isoformat(),
        }

        return args

    def determine_graph_default_range(self, query_string_high, query_string_low, high_ask, low_bid, open_orders):
        """
        If a range is specified in the query string, use that.
        """

        # Defaults
        high = high_ask + Money(self.DEFAULT_WIDTH, self.price_currency)
        low = low_bid - Money(self.DEFAULT_WIDTH, self.price_currency)

        # Make sure we get open order prices on the graph.
        if len(open_orders) > 0:
            order_prices = []

            for exchange, exchange_orders in open_orders.items():
                for o in exchange_orders:
                    order_prices.append(o[0])

            order_prices = sorted(order_prices)

            lowest_order = order_prices[0]
            highest_order = order_prices[-1]

            # Is the parameter right here?
            if lowest_order < low:
                low = Money(str(lowest_order * 0.99), self.price_currency)

            if highest_order > high:
                high = Money(str(highest_order * 1.01), self.price_currency)

        # Query string overrides.
        if query_string_high is not None:
            high = Money(query_string_high, self.price_currency)

        if query_string_low is not None:
            low = Money(query_string_low, self.price_currency)

        return high, low

    def get_orderbook_levels(self, gds_db, trading_pairs, at_time, selected_exchanges):
        orderbooks = self.get_all_orderbooks(
            self.trading_pairs,
            self.gds_db,
            at_time,
        )

        levels_for_exchange, low_bid, high_ask = self.get_levels_from_orderbooks(
            orderbooks,
            selected_exchanges=selected_exchanges,
        )

        return levels_for_exchange, low_bid, high_ask

    def generate_null_data(self):
        open_orders_for_exchange = {}
        orderbooks = defaultdict(lambda x: [])
        levels_for_exchange = {e: [] for e in self.trading_pair_names}

        args = {
            'gds_active': False,
            'levels_for_exchange': levels_for_exchange,
            'selected_exchanges': [''],
            'open_orders_for_exchange': open_orders_for_exchange,
            'high': Money('1', 'USD'),
            'low': Money('0', 'USD'),
            'pair_name': self.pair_name,
            'show_orders': False,
            'at_time': None,
        }

        return args

    def get_open_orders_by_exchange(self, at_time=None):
        """
        Get the orders in the trading database that are currently open, or optionally
        were open at some point in the past.
        """
        orders = []

        if at_time is None:
            orders = self.trading_db.query(Order)\
                .filter(Order.status == 'OPEN')\
                .filter(Order._exchange_name.in_(self.trading_pair_names))\
                .order_by(Order._price * Order.exchange_rate)\
                .all()
        else:
            orders = self.trading_db.query(Order)\
                .filter(Order._exchange_name.in_(self.trading_pair_names))\
                .filter(Order.time_created < at_time)\
                .filter(Order.time_executed > at_time)\
                .order_by(Order._price * Order.exchange_rate)\
                .all()

        open_orders = defaultdict(lambda: [])

        for order in orders:
            price = float(order.price.to('USD').amount)
            volume = float(order.volume.amount)
            order_type = order.order_type

            open_orders[order._exchange_name].append([price, volume, order_type])

        return open_orders

    def get_all_orderbooks(self, exchange_objects, gds_db, at_time=None):
        orderbooks = defaultdict(lambda x: [])

        for e in exchange_objects:
            orderbook = self.get_orderbook_for_exchange(gds_db, e, at_time)

            orderbooks[e.name] = orderbook

        return orderbooks

    def get_orderbook_for_exchange(self, gds_db, exchange, at_time=None):
        db_orderbook = None

        if at_time:
            db_orderbook = self.get_orderbook_for_exchange_at_time(
                gds_db,
                exchange,
                at_time,
            )
        else:
            db_orderbook = self.get_latest_orderbook_for_exchange(gds_db, exchange)

        if db_orderbook is None:
            return None

        raw_orderbook = {
            'bids': db_orderbook.bids,
            'asks': db_orderbook.asks,
        }

        orderbook = exchange.parse_orderbook(raw_orderbook, cached_orders=True)

        return orderbook

    def get_latest_orderbook_for_exchange(self, gds_db, exchange):
        orderbook = None

        if self.no_lookback_limit is True:
            orderbook = gds_db.query(Orderbook)\
                .filter(Orderbook.exchange == exchange.name.upper())\
                .order_by(Orderbook.timestamp.desc())\
                .first()
        else:
            # We only show orderbooks from the last hour.
            now = Delorean()
            latest_orderbook_time = now.naive
            earliest_orderbook_time = now.last_hour(1).naive

            orderbook = gds_db.query(Orderbook)\
                .filter(Orderbook.exchange == exchange.name.upper())\
                .filter(Orderbook.timestamp > earliest_orderbook_time)\
                .filter(Orderbook.timestamp <= latest_orderbook_time)\
                .order_by(Orderbook.timestamp.desc())\
                .first()

        return orderbook

    def get_orderbook_for_exchange_at_time(self, gds_db, exchange, at_time):
        """
        TODO:
          - this should also have some time bounds on it. And shouldn't it be the
            first orderbook before `at_time`?
        """
        orderbook = gds_db.query(Orderbook)\
            .filter(Orderbook.exchange == exchange.name.upper())\
            .filter(Orderbook.timestamp > at_time)\
            .order_by(Orderbook.timestamp.asc())\
            .first()

        return orderbook

    def get_levels_from_orderbooks(self, orderbooks, selected_exchanges=[]):
        levels_for_exchange = {}

        low_bid = Money(self.infinity, self.price_currency)
        high_ask = Money(self.zero, self.price_currency)

        for exchange_name, orderbook in orderbooks.items():
            if orderbook == None:
                levels_for_exchange[exchange_name] = None
                continue

            levels, top_bid, top_ask = self.get_levels_from_orderbook_in_usd(orderbook)

            if top_bid < low_bid and exchange_name.upper() in selected_exchanges:
                low_bid = top_bid

            if top_ask > high_ask and exchange_name.upper() in selected_exchanges:
                high_ask = top_ask

            levels_for_exchange[exchange_name] = levels

        return levels_for_exchange, low_bid, high_ask

    def get_levels_from_orderbook_in_usd(self, orderbook):
        levels = []
        total_bid_volume = Money('0', self.volume_currency)
        total_ask_volume = Money('0', self.volume_currency)

        for order in orderbook['bids']:
            price_in_btc = order.price

            total_bid_volume += order.volume
            levels.append([price_in_btc, total_bid_volume])

        for order in orderbook['asks']:
            price_in_btc = order.price

            total_ask_volume += order.volume
            levels.append([price_in_btc, total_ask_volume])

        # Extra operations to massage the data a little bit.

        # Set fake orders at zero volume just on the inside of the top bid and ask
        # so that we can see the inside spread on graphs.
        top_bid = orderbook['bids'][0].price
        levels.append([
            top_bid + Money(self.infinitisimal, self.price_currency),
            Money('0', self.volume_currency),
        ])

        top_ask = orderbook['asks'][0].price
        levels.append([
            top_ask - Money(self.infinitisimal, self.price_currency),
            Money('0', self.volume_currency),
        ])

        levels = sorted(levels, key=lambda l: l[0].amount)

        return levels, top_bid, top_ask

    def format_levels_for_graphing(self, exchange_levels):
        formatted_levels = {}

        for exchange_name, levels in exchange_levels.items():
            if levels == None:
                formatted_levels[exchange_name] = []
                continue

            formatted_levels[exchange_name] = self.format_levels(levels)

        return formatted_levels

    def format_levels(self, levels):
        formatted_levels = []

        for price, volume in levels:
            formatted_levels.append([
                float(price.amount),
                float(volume.amount),
            ])

        if len(formatted_levels) < 1:
            return []

        return formatted_levels




import json
import os
import requests

from cdecimal import *
from delorean import Delorean, epoch

from gryphon.lib.session import get_a_redis_connection
from gryphon.lib.logger import get_logger
from gryphon.lib.cache import cache_me

logger = get_logger(__name__)


MISSING_ENV_VAR_ERROR = """Requires environment variables: EXCHANGE_RATE_APP_ID"""


class USDCurrencyConverter(object):
    @staticmethod
    def get_rates_from_redis():
        exchange_rate_age_threshold = 60*60 # 1hr
        
        try:
            r = get_a_redis_connection()
            exchange_rates_string = r.get('emerald_havoc_exchange_rates')
            logger.info(exchange_rates_string[:100])

            if exchange_rates_string:
                exchange_rates_dict = json.loads(
                    exchange_rates_string,
                    parse_float=Decimal,
                )

                timestamp = exchange_rates_dict['timestamp']
                now_timestamp = Delorean().epoch

                if timestamp + exchange_rate_age_threshold > now_timestamp:
                    return exchange_rates_dict['exchange_rates']['rates']
        except Exception as e:
            logger.info('Could not establish redis exchange rate.')
            return None
        
    
    @staticmethod
    def get_rates_from_http():
        try:
            app_id = os.environ['EXCHANGE_RATE_APP_ID']
        except:
            raise Exception(MISSING_ENV_VAR_ERROR)
        
        url = 'https://openexchangerates.org/api/latest.json?app_id=%s&show_alternative=true' % app_id

        r = requests.get(url).json(parse_float=Decimal)

        return r['rates']

    @staticmethod
    @cache_me(time=60) # 1min
    def _all_rates():
        rates = USDCurrencyConverter.get_rates_from_redis()

        if not rates:
            rates = USDCurrencyConverter.get_rates_from_http()

        return rates
    
    @staticmethod
    def rate(currency_code):
        rates = USDCurrencyConverter._all_rates()
        rate = rates[currency_code]
        return rate

    @staticmethod
    @cache_me(time=600) # 10min
    def _historical_rates(year, month, day):
        """
        We can't cache this for longer because we check "historical" rate for today,
        which can change throughout the day.
        """
        try:
            app_id = os.environ['EXCHANGE_RATE_APP_ID']
        except:
            raise Exception(MISSING_ENV_VAR_ERROR)

        url = 'https://openexchangerates.org/api/historical/%d-%02d-%02d.json?app_id=%s&show_alternative=true' % (year, month, day, app_id)

        r = requests.get(url).json(parse_float=Decimal)

        return r['rates']

    @staticmethod
    def historical_rate(currency_code, date):
        historical_rates = USDCurrencyConverter._historical_rates(
            date.year,
            date.month,
            date.day,
        )

        rate = historical_rates[currency_code]

        return rate




import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)


def movingaverage(values,window):
    weigths = np.repeat(1.0, window)/window
    smas = np.convolve(values, weigths, 'valid')
    return smas # as a numpy array



def EMV(d,c,h,l,o,v,tf):

    x=1
    OnepEMV = []
    while x < len(c):
        movement = ( ((h[x]+l[x])/2) - ((h[x-1]+l[x-1])/2) )
        boxr = ( (v[x]/1000000.00)/ (h[x]-l[x]) )
        OnepEMVs = movement / boxr
        OnepEMV.append(OnepEMVs)
        print OnepEMVs
        x += 1

    tfEMV = movingaverage(OnepEMV,tf)

    print len(tfEMV)
    print len(d[tf:])

    return d[tf:],tfEMV

EMV(date,closep,highp,lowp,openp,volume,14)



"""
Functions that run checks on an exchange account to verify that our ledger is consistent
with the exchange state.
"""

from cdecimal import Decimal
import termcolor as tc

from sqlalchemy.orm import joinedload

from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.gryphonfury import positions
from gryphon.lib.money import Money
from gryphon.lib.logger import get_logger
from gryphon.lib.models.order import Order
from gryphon.lib.models.transaction import Transaction

logger = get_logger(__name__)

# Audit type constants.
ORDER_AUDIT = 'order'
VOLUME_BALANCE_AUDIT = 'volume_balance'
FIAT_BALANCE_AUDIT = 'fiat_balance'
POSITION_CACHE_AUDIT = 'position_cache'
LEDGER_AUDIT = 'ledger'
ALL_AUDITS = [
    ORDER_AUDIT,
    VOLUME_BALANCE_AUDIT,
    FIAT_BALANCE_AUDIT,
    LEDGER_AUDIT,
]

class AuditException(Exception):
    pass


class OrderAuditException(AuditException):
    def __init__(self, exchange, failed_order_data):
        message = '[%s] %s order(s) failed the audit' % (
            exchange.name,
            len(failed_order_data),
        )

        super(OrderAuditException, self).__init__(message)
        self.exchange = exchange
        self.failed_order_data = failed_order_data
        self.message = message


def audit(exchange_key):
    db = session.get_a_trading_db_mysql_session()

    try:
        if exchange_key:
            exchange = exchange_factory.make_exchange_from_key(exchange_key)
            exchanges = [exchange]
        else:
            exchanges = exchange_factory.all_exchanges()

        for exchange in exchanges:
            open_orders = exchange.get_open_orders()
            if open_orders:
                logger.info(tc.colored(
                    'Cannot audit: there are open orders on %s' % (
                        exchange.friendly_name,
                    ),
                    'red',
                ))

                return

            exchange_data = exchange.exchange_account_db_object(db)

            try:
                order_audit(db, exchange, skip_recent=0)
                logger.info(tc.colored('Order Audit passed', 'green'))
            except OrderAuditException as e:
                logger.info(tc.colored('Order Audit: %s' % e.message, 'red'))

                for datum in e.failed_order_data:
                    logger.info(tc.colored(datum[1], 'red'))

            # Call this here so we don't call it twice in the two balance audits.
            exchange_balance = exchange.get_balance()

            try:
                volume_balance_audit(
                    exchange,
                    exchange_data,
                    exchange_balance=exchange_balance,
                )

                logger.info(tc.colored('BTC Audit passed', 'green'))
            except AuditException as e:
                logger.info(tc.colored(
                    '%s Audit: %s' % (exchange.volume_currency, e.message),
                    'red',
                ))

            try:
                fiat_balance_audit(
                    exchange,
                    exchange_data,
                    exchange_balance=exchange_balance,
                )

                logger.info(tc.colored('Fiat Audit passed', 'green'))
            except AuditException as e:
                logger.info(tc.colored('Fiat Audit: %s' % e.message, 'red'))

            try:
                position_cache_audit(db, exchange_data)
                logger.info(tc.colored('Position Cached Audit passed', 'green'))
            except AuditException as e:
                logger.info(tc.colored('Position Cached Audit: %s' % e.message, 'red'))

            try:
                ledger_audit(exchange_data)
                logger.info(tc.colored('Ledger Audit passed', 'green'))
            except AuditException as e:
                logger.info(tc.colored('Ledger Audit: %s' % e.message, 'red'))

    finally:
        session.commit_mysql_session(db)
        db.remove()


def close_enough(a, b, tolerance):
    return abs(a - b) <= tolerance


def order_audit(db, exchange, skip_recent=0, tolerance=Decimal('0')):
    audit_data = exchange.get_order_audit_data(skip_recent=skip_recent)
    exchange_order_ids = audit_data.keys()

    if not exchange_order_ids:  # No point hitting the database to check for no trades.
        return

    db_orders = db.query(Order)\
        .filter(Order._exchange_name == exchange.name)\
        .filter(Order.exchange_order_id.in_(exchange_order_ids))\
        .options(joinedload('trades'))\
        .all()

    db_order_hash = {}

    for db_order in db_orders:
        db_order_hash[db_order.exchange_order_id] = db_order.volume_filled

    failed_order_data = []

    for exchange_order_id, exchange_volume_filled in audit_data.iteritems():
        try:
            db_volume_filled = db_order_hash[exchange_order_id]

            if not close_enough(db_volume_filled, exchange_volume_filled, tolerance):
                audit_message = 'Order volume mismatch on #%s: %.8f != %.8f' % (
                    exchange_order_id,
                    exchange_volume_filled.amount,
                    db_volume_filled.amount,
                )

                failed_order_datum = (exchange_order_id, audit_message)
                failed_order_data.append(failed_order_datum)
        except KeyError:
            raise AuditException('Order #%s not found in database' % exchange_order_id)

    if failed_order_data:
        raise OrderAuditException(exchange, failed_order_data)

    return audit_data



def fiat_balance_audit(exchange, exchange_data, tolerance=Decimal('0'), execute=True, exchange_balance=None):
    """
    This and volume_balance_audit return the exact same data: the exchange's crypto and
    fiat balance.
    """

    balance_audit_data = balance_audit(
        exchange.currency,
        exchange,
        exchange_data,
        tolerance=tolerance,
        execute=execute,
        exchange_balance=exchange_balance,
    )

    return balance_audit_data


def volume_balance_audit(exchange, exchange_data, tolerance=Decimal('0'), execute=True, exchange_balance=None):
    balance_audit_data = balance_audit(
        exchange.volume_currency,
        exchange,
        exchange_data,
        tolerance=tolerance,
        execute=execute,
        exchange_balance=exchange_balance,
    )


    return balance_audit_data


def balance_audit(currency, exchange, exchange_data, tolerance=Decimal('0'), execute=True, exchange_balance=None):
    logger.debug('Auditing %s' % currency)

    if exchange_balance is None:
        exchange_balance = exchange.get_balance()

    exchange_amount = exchange_balance[currency]

    db_amount = exchange_data.balance[currency]
    db_amount = exchange.process_db_balance_for_audit(db_amount)

    # Check for incoming deposits.
    if not close_enough(db_amount, exchange_amount, tolerance):
        logger.debug('Balance audit failed, looking for pending deposits')

        balance_change = exchange_amount - db_amount
        success = exchange_data.deposit_landed(balance_change, tolerance=tolerance)

        if success:
            # Update balance to include landed deposits.
            db_amount = exchange_data.balance[currency]
        else:
            raise AuditException(
                'Our balance: %s != Exchange balance: %s' % (
                    db_amount,
                    exchange_amount,
                ))

    # If we're still not close enough after checking for incoming deposits.
    if not close_enough(db_amount, exchange_amount, tolerance):
        raise AuditException(
            'Our balance: %s != Exchange balance: %s' % (db_amount, exchange_amount),
        )

    # If we get this far, we passed the audit (maybe with some drift).
    # Drift measures our balance's departure from reality (the exchange), so a positive     # drift means we thought we had a higher balance than reality (i.e. an undocumented
    # loss).

    drift = db_amount - exchange_amount

    # If running in no-execute mode, we don't fix drift, and it is also not recorded
    # in audits.
    if execute and drift:
        record_drift(drift, exchange_data)

    return exchange_balance


def record_drift(drift, exchange_data):
    if drift > 0:
        transaction_type = Transaction.WITHDRAWL
    else:
        transaction_type = Transaction.DEPOSIT

    amount = abs(drift)

    logger.debug('Adding drift %s for %s' % (transaction_type, repr(amount)))

    drift_transaction = Transaction(
        transaction_type,
        Transaction.IN_TRANSIT,
        amount,
        exchange_data,
        {'drift': True},
    )

    drift_transaction.complete()


def balance_equality(balance_a, balance_b):
    """
    Because of how gryphon.lib.assets works currently, ledger_balance does not include
    keys for currencies which it has no transactions or trades for. This causes the
    normal dictionary equality operator to return false on a case where we really do
    have the right data. An example like this: {'USD': 50, 'BTC': 0} == {'USD': 50},
    with the latter being the ledger balance.

    It's out of scope for the current work to make modifications to the very-critical
    gryphon.lib.assets, but this is causing audits to fail on fresh databases, so we
    are fixing it here with the intention of a later fix to the core lib.
    """

    currencies = set(balance_a.keys() + balance_b.keys())

    for currency in currencies:
        if currency in balance_a and currency in balance_b:
            if balance_a[currency] == balance_b[currency]:
                continue
            else:
                return False
        elif currency not in balance_a:
            if balance_b[currency] == Money('0', currency):
                continue
            else:
                return False
        elif currency not in balance_b:
            if balance_a[currency] == Money('0', currency):
                continue
            else:
                return False

    return True


def ledger_audit(exchange_data):
    ledger_balance = exchange_data.ledger_balance()

    db_balance = exchange_data.balance

    if balance_equality(db_balance, ledger_balance) is not True:
        raise AuditException('DB balance: %s != Ledger balance: %s' % (
            db_balance,
            ledger_balance,
        ))


def position_cache_audit(db, exchange_data):
    multi_position = positions.fast_position(db, exchange_name=exchange_data.name)
    cached_position = exchange_data.multi_position_cache

    if multi_position != cached_position:
        raise AuditException('Cached position: %s != DB position: %s' % (
            cached_position, multi_position,
        ))




"""
Tests for gryphon.lib.arbitrage.

TODO:
- Test fee/profit number functionality. This is a bit more than trivial because it's
  not guaranteed that our fee levels on a given exchange won't change.
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import unittest
import sure

from gryphon.lib import order_sliding
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange


class TestOrderSliding(unittest.TestCase):
    def setUp(self):
        self.bitstamp = BitstampBTCUSDExchange()

        bids1 = [
            Order(Money('1000', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.BID),
            Order(Money('500', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.BID),
        ]

        asks1 = [
            Order(Money('1001', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.ASK),
            Order(Money('1501', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.ASK),
        ]

        self.basic_a = {'bids': bids1, 'asks': asks1}

        bids2 = [
            Order(Money('1000', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.BID),
            Order(Money('900', 'USD'), Money('0.01', 'BTC'), self.bitstamp, Consts.BID),
            Order(Money('500', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.BID),
        ]

        asks2 = [
            Order(Money('1001', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.ASK),
            Order(Money('1100', 'USD'), Money('0.01', 'BTC'), self.bitstamp, Consts.BID),
            Order(Money('1501', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.ASK),
        ]

        self.basic_b = {'bids': bids2, 'asks': asks2}

    def tearDown(self):
        pass
  
    def test_trivial_bid(self):
        mode = Consts.BID
        initial_price = Money('999', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('500.01', 'USD'))

    def test_trivial_ask(self):
        mode = Consts.ASK
        initial_price = Money('1002', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('1500.99', 'USD'))

    def test_max_slide_bid(self):
        mode = Consts.BID
        initial_price = Money('999', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = Money('100', 'USD')

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('899', 'USD'))

    def test_max_slide_ask(self):
        mode = Consts.ASK
        initial_price = Money('1002', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = Money('100', 'USD')

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('1102', 'USD'))

    def test_jump_bid(self):
        mode = Consts.BID
        initial_price = Money('999', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('100', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('600', 'USD'))

    def test_jump_ask(self):
        mode = Consts.ASK
        initial_price = Money('1002', 'USD')
        orderbook = self.basic_a
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('100', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('1401', 'USD'))

    def test_ignore_bid(self):
        mode = Consts.BID
        initial_price = Money('999', 'USD')
        orderbook = self.basic_b
        ignore_volume = Money('0.1', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('500.01', 'USD'))

    def test_ignore_ask(self):
        mode = Consts.ASK
        initial_price = Money('1002', 'USD')
        orderbook = self.basic_b
        ignore_volume = Money('0.1', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('1500.99', 'USD'))

    def test_do_not_ignore_bid(self):
        mode = Consts.BID
        initial_price = Money('999', 'USD')
        orderbook = self.basic_b
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('900.01', 'USD'))

    def test_do_not_ignore_ask(self):
        mode = Consts.ASK
        initial_price = Money('1002', 'USD')
        orderbook = self.basic_b
        ignore_volume = Money('0.001', 'BTC')
        jump = Money('0.01', 'USD')
        max_slide = None

        new_price = order_sliding.slide_order(
            mode,
            initial_price,
            orderbook,
            ignore_volume,
            jump,
            max_slide,
        )

        new_price.should.equal(Money('1099.99', 'USD'))


class TestOrderPriceLogic(unittest.TestCase):
    def setUp(self):
        #o = Order(Money('600', 'USD'), Money('1', 'BTC'), self.bitstamp, Consts.BID),
        pass

    def tearDown(self):
        pass

    def test_widen_bid(self):
        price = Money('1', 'USD')
        change = Money('0.1', 'USD')
        mode = Consts.BID

        result = order_sliding.widen_price(mode, price, change)

        result.should.equal(Money('0.9', 'USD'))

    def test_widen_ask(self):
        price = Money('1', 'USD')
        change = Money('0.1', 'USD')
        mode = Consts.ASK

        result = order_sliding.widen_price(mode, price, change)

        result.should.equal(Money('1.1', 'USD'))

    def test_narrow_bid(self):
        price = Money('1', 'USD')
        change = Money('0.1', 'USD')
        mode = Consts.BID

        result = order_sliding.narrow_price(mode, price, change)

        result.should.equal(Money('1.1', 'USD'))

    def test_narrow_ask(self):
        price = Money('1', 'USD')
        change = Money('0.1', 'USD')
        mode = Consts.ASK

        result = order_sliding.narrow_price(mode, price, change)

        result.should.equal(Money('0.9', 'USD'))

    def test_widen_bid_b(self):
        price = Money('1e6', 'BCH')
        change = Money('100', 'BCH')
        mode = Consts.BID

        result = order_sliding.widen_price(mode, price, change)

        result.should.equal(Money('999900', 'BCH'))

    def test_widen_ask_b(self):
        price = Money('1e6', 'BCH')
        change = Money('100', 'BCH')
        mode = Consts.ASK

        result = order_sliding.widen_price(mode, price, change)

        result.should.equal(Money('1000100', 'BCH'))

    def test_narrow_bid_b(self):
        price = Money('1e6', 'BCH')
        change = Money('100', 'BCH')
        mode = Consts.BID

        result = order_sliding.narrow_price(mode, price, change)

        result.should.equal(Money('1000100', 'BCH'))

    def test_narrow_ask(self):
        price = Money('1e6', 'BCH')
        change = Money('100', 'BCH')
        mode = Consts.ASK

        result = order_sliding.narrow_price(mode, price, change)

        result.should.equal(Money('999900', 'BCH'))

    def test_deeper_price_bid(self):
        price_a = Money('1', 'USD')
        price_b = Money('0.9', 'USD')
        mode = Consts.BID

        result = order_sliding.is_deeper_price(mode, price_a, price_b)

        result.should.equal(False)

        result = order_sliding.is_deeper_price(mode, price_b, price_a)

        result.should.equal(True)

    def test_deeper_price_ask(self):
        price_a = Money('1', 'USD')
        price_b = Money('0.9', 'USD')
        mode = Consts.ASK

        result = order_sliding.is_deeper_price(mode, price_a, price_b)

        result.should.equal(True)

        result = order_sliding.is_deeper_price(mode, price_b, price_a)

        result.should.equal(False)

    def test_deeper_price_bid_b(self):
        price_a = Money('1000', 'ETH')
        price_b = Money('200', 'ETH')
        mode = Consts.BID

        result = order_sliding.is_deeper_price(mode, price_a, price_b)

        result.should.equal(False)

        result = order_sliding.is_deeper_price(mode, price_b, price_a)

        result.should.equal(True)

    def test_deeper_price_ask_b(self):
        price_a = Money('1000', 'ETH')
        price_b = Money('200', 'ETH')
        mode = Consts.ASK

        result = order_sliding.is_deeper_price(mode, price_a, price_b)

        result.should.equal(True)

        result = order_sliding.is_deeper_price(mode, price_b, price_a)

        result.should.equal(False)




from collections import defaultdict
import json

from delorean import Delorean

from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money
from gryphon.lib.models.event import Event
from gryphon.lib.models.exchange import Balance


def get_balance_time_series_from_audits(audits):
    fiat_balances = []
    btc_balances = []
    
    for audit in audits:
        timestamp = int(Delorean(audit.time_created, "UTC").epoch) * 1000

        # load the fiat and btc balances from the audit data
        data = json.loads(audit.data)

        try:
            # Old data format from before Jan 28, 2015
            raw_fiat = data['exchange_balance']['fiat_available']
            raw_btc = data['exchange_balance']['btc_available']
            fiat = Money.loads(raw_fiat).to("USD")
            btc = Money.loads(raw_btc)

        except KeyError:
            # New data format from after Jan 28, 2015
            try:
                balance_data = data['balance_data']
            except KeyError:
                continue

            # convert to Money objects
            for currency, balance_str in balance_data.iteritems():
                balance_data[currency] = Money.loads(balance_str)

            balance = Balance(balance_data)
            fiat = balance.fiat().to('USD')
            btc = balance['BTC']

        fiat_datapoint = [
            timestamp,
            str(fiat.amount),
        ]

        fiat_balances.append(fiat_datapoint)

        btc_datapoint = [
            timestamp,
            str(btc.amount),
        ]

        btc_balances.append(btc_datapoint)

    return fiat_balances, btc_balances


def get_audits_for_exchange(db, exchange_name, start_time=None, end_time=None, data_filter=None):
    audit_query = db\
        .query(Event)\
        .filter(Event.event_type == 'AUDIT')\
        .filter(Event.exchange_name == exchange_name)\
        .filter(Event.time_created > start_time)\
        .filter(Event.time_created < end_time)

    if data_filter:
        audit_query = audit_query.filter(Event.data.contains(data_filter))

    audits = audit_query.all()

    return audits


def get_all_audits_in_period(db, start_time, end_time, data_filter=None):
    audit_query = db\
        .query(Event)\
        .filter(Event.event_type == 'AUDIT')\
        .filter(Event.time_created > start_time)\
        .filter(Event.time_created < end_time)

    if data_filter:
        audit_query = audit_query.filter(Event.data.contains(data_filter))

    audits = audit_query.all()

    return audits


def get_balance_time_series_for_exchange(db, exchange_name, start_time, end_time):
    audits = get_audits_for_exchange(db, exchange_name, start_time, end_time)
    series = get_balance_time_series_from_audits(audits)

    return series


def get_drift_from_audits(audits):
    drift_by_currency = Balance()

    for audit in audits:
        if 'drift' in audit.data:
            data = json.loads(audit.data)

            for currency, str_amount in data['drift'].iteritems():
                drift_by_currency += Money.loads(str_amount)

    return drift_by_currency


def get_total_drift_in_period(db, start_time, end_time):
    audits = get_all_audits_in_period(db, start_time, end_time, data_filter='drift')

    return get_drift_from_audits(audits)


def get_drift_for_exchange_in_period(db, exchange_name, start_time, end_time):
    audits = get_audits_for_exchange(db, exchange_name, start_time, end_time, data_filter='drift')

    return get_drift_from_audits(audits)



"""
A bare-bones wrapper class that allows us to turn off sentry logging if we don't want
to use it (e.g. in testing).
"""

import os

import raven


class Sentry(object):
    def __init__(self, is_active=False):
        self.is_active = is_active
        self.sentry = None

        if self.is_active is True:
            self.sentry = raven.Client(os.environ['SENTRY_URL'])

    def captureException(self):
        if self.is_active is True:
            self.sentry.captureException()



import numpy as np
from cdecimal import Decimal

def MovingAverage(values, timestamps, window_time=300):
    assert len(values) == len(timestamps)
    ma = []
    values.reverse()
    timestamps.reverse()
    for i in range(len(timestamps)):
        noted_timestamp = timestamps[i]
        relevant_values = []
        relevant_timestamps = []
        for j in range(len(timestamps)):
            index = j+i
            if index > len(timestamps)-1:
                break 
            elif noted_timestamp - timestamps[index] < window_time:
                relevant_values.append(values[index])
                relevant_timestamps.append(noted_timestamp - timestamps[index])
            else:
                break
        
        weights = [Decimal(1)/len(relevant_values) for a in relevant_values]
        weighted_value = 0
        for i in range(len(relevant_values)):
            rv = relevant_values[i]
            weight = weights[i]
            weighted_value += rv*weight
        ma.append(weighted_value)
    ma.reverse()
    return ma



# -*- coding: utf-8 -*-
import time
import os
import datetime
import requests
import hmac
import urllib
import hashlib
import base64
import json
from delorean import Delorean, parse
import urllib
from collections import OrderedDict

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money
from base import *
from exceptions import *
from exchange_order import Order
from gryphon.lib.models.exchange import Balance
from gryphon.lib.models.datum import DatumRecorder

from cdecimal import *
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class ButtercoinExchange(Exchange):
    def __init__(self, session=None, use_cached_orderbook=False):
        super(ButtercoinExchange, self).__init__(session)
        self.name = u'BUTTERCOIN'
        self.friendly_name = u'Buttercoin'
        self.base_url = 'https://api.buttercoin.com/v1'
        self.currency = "USD"
        self.fee = Decimal("0")
        self.bid_string = "buy"
        self.ask_string = "sell"
        self.use_cached_orderbook = use_cached_orderbook



import pyximport; pyximport.install()

from gryphon.lib.exchange.itbit_btc_usd import ItbitBTCUSDExchange
from gryphon.tests.exceptional.exchange_wrappers.auth_methods import ExchangeAuthMethodsTests


class TestItbitBTCUSDAuthMethods(ExchangeAuthMethodsTests):
    def setUp(self):
        self.exchange = ItbitBTCUSDExchange()



"""
This simple strategy mixin rebalances bitcoin balances between multiple exchanges at the
end of every nth tick, with n set by the 'bitcoin_rebalance_tick' config variable.
"""

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.lib.crypto_balancing import bitcoin_rebalance


class PostTickRebalancing(Strategy):
    def post_tick(self, tick_count):
        if tick_count % self.config['bitcoin_rebalance_tick'] == 0:
            bitcoin_rebalance(self.db, self.exchange_data, self, self.execute)

        super(PostTickRebalancing, self).post_tick(tick_count)



# -*- coding: utf-8 -*-
import pyximport; pyximport.install()
import logging
import logging.handlers
import os
import sys

from raven import Client
from twisted.internet import reactor
from twisted.python import log
from twisted.web import client

from gryphon.data_service.auditor_task import AuditorTask
from gryphon.data_service.orderbook_poll_task import OrderbookPollTask
from gryphon.data_service.trades_poll_task import TradesPollTask
from gryphon.data_service.utilities_poll_task import UtilitiesPollTask
from gryphon.data_service.volume_poll_task import VolumePollTask


# Set up Logging.
logger = logging.getLogger('emerald-havoc-producer')

if 'SYSLOG_ADDRESS' in os.environ:
    syslog = logging.handlers.SysLogHandler(
        address=(
            os.environ['SYSLOG_ADDRESS'],
            int(os.environ['SYSLOG_PORT'])
        )
    )

    syslog.setLevel(logging.DEBUG)
    format_str = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    syslog.setFormatter(logging.Formatter(format_str))
    logger.addHandler(syslog)


# Set up Exception Handling.
if 'SENTRY_DSN' in os.environ:
    twisted_sentry_client = Client(dsn='twisted+%s' % os.environ['SENTRY_DSN'])

    def logToSentry(event):
        if event.get('isError'):
            if 'failure' in event:
                f = event['failure']

                twisted_sentry_client.captureException(
                    (f.type, f.value, f.getTracebackObject()),
                )
            else:
                twisted_sentry_client.captureMessage(event['message'])

    log.addObserver(logToSentry)


def main(*args):
    exchanges = sys.argv[1:]
    OrderbookPollTask(exchanges=exchanges).start_task()

    # Hack for now so we don't start unimplemented trade pollers on the eth_btc pairs.
    non_orderbook_exchanges = [
        e for e in exchanges if all([
            x not in e for x in ['ETH', 'BCH', 'EUR']
        ])
    ]

    logger.info(non_orderbook_exchanges)

    TradesPollTask(exchanges=non_orderbook_exchanges).start_task()
    AuditorTask(exchanges=non_orderbook_exchanges).start_task()
    UtilitiesPollTask().start_task()
    VolumePollTask(exchanges=non_orderbook_exchanges).start_task()

    observer = log.PythonLoggingObserver('emerald-havoc-producer')
    observer.start()
    client._HTTP11ClientFactory.noisy = False

    reactor.run()

if __name__ == "__main__":
    main()



from gryphon.lib.exchange.coinbase_btc_usd import CoinbaseBTCUSDExchange
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money

logger = get_logger(__name__)


class CoinbaseBTCCADExchange(CoinbaseBTCUSDExchange):
    def __init__(self, session=None, use_cached_orderbook=False):
        super(CoinbaseBTCCADExchange, self).__init__(session)
        self.name = u'COINBASE_BTC_CAD'
        self.friendly_name = u'Coinbase BTC-CAD'
        self.base_url = 'https://api.exchange.coinbase.com'
        self.currency = 'CAD'
        self.use_cached_orderbook = use_cached_orderbook
        self.product_id = 'BTC-CAD'

    def load_creds(self):
        try:
            self.api_key
            self.secret
            self.passphrase
        except AttributeError:
            self.api_key = self.load_env('COINBASE_BTC_CAD_API_KEY')
            self.passphrase = self.load_env('COINBASE_BTC_CAD_API_PASSPHRASE')
            self.secret = self.load_env('COINBASE_BTC_CAD_API_SECRET')

    def load_wallet_creds(self):
        try:
            self.wallet_api_key
            self.wallet_api_secret
            self.wallet_id
        except AttributeError:
            self.wallet_api_key = self.load_env('COINBASE_BTC_CAD_WALLET_API_KEY')
            self.wallet_api_secret = self.load_env('COINBASE_BTC_CAD_WALLET_API_SECRET')
            self.wallet_id = self.load_env('COINBASE_BTC_CAD_WALLET_ID')

    def load_fiat_account_id(self):
        return self.load_env('COINBASE_BTC_CAD_FIAT_ACCOUNT_ID')

    def fiat_deposit_fee(self, deposit_amount):
        return Money('1', 'CAD')



# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict
import hashlib
import hmac
import json
import time

import cdecimal
from cdecimal import Decimal
from delorean import Delorean, parse

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

logger = get_logger(__name__)


class BitfinexBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(BitfinexBTCUSDExchange, self).__init__(session)

        self.name = u'BITFINEX_BTC_USD'
        self.friendly_name = u'Bitfinex BTC-USD'
        self.base_url = 'https://api.bitfinex.com/v1'
        self.currency = 'USD'
        self.bid_string = 'buy'
        self.ask_string = 'sell'

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.002')
        self.limit_order_fee = Decimal('0.001')
        self.fee = Decimal('0.00')
        self.fiat_balance_tolerance = Money('0.01', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0.001', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    def req(self, req_method, url, **kwargs):
        req = super(BitfinexBTCUSDExchange, self).req(req_method, url, **kwargs)
        return req

    def resp(self, req):
        response = super(BitfinexBTCUSDExchange, self).resp(req)
        if 'message' in response:
            errors_string = str(response['message'])
            if 'not enough balance' in errors_string:
                raise exceptions.InsufficientFundsError()
            elif 'Order could not be cancelled' in errors_string:
                raise exceptions.CancelOrderNotFoundError()
            elif 'Nonce is too small' in errors_string:
                raise exceptions.NonceError()
            else:
                raise exceptions.ExchangeAPIErrorException(self, errors_string)

        return response

    def all_trades(self, until=None):
        req = self.all_trades_req(until)
        return self.all_trades_resp(req)

    def all_trades_req(self, until=None):
        payload = {
            'symbol': 'btcusd',
            'limit_trades': 500,
        }

        if until:
            payload['until'] = str(until)

        return self.req('post', '/mytrades', data=payload)

    def all_trades_resp(self, req):
        return self.resp(req)

    def trades_for_orders(self, order_ids):
        req = self.trades_for_orders_req()
        return self.trades_for_orders_resp(req, order_ids)

    def trades_for_orders_req(self):
        return self.all_trades_req()

    def trades_for_orders_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]
        trades = self.all_trades_resp(req)

        matching_trades = {}

        for trade in trades:
            oid = unicode(trade['order_id'])
            if oid in order_ids:
                if not oid in matching_trades:
                    matching_trades[oid] = []
                matching_trades[oid].append(trade)
        return matching_trades

    ######## Common Exchange Methods ########

    # modifies request_args
    def auth_request(self, req_method, url, request_args):
        try:
            self.api_key
            self.secret
        except AttributeError:
            self.api_key = self.load_env('BITFINEX_BTC_USD_API_KEY')
            self.secret = self.load_env('BITFINEX_BTC_USD_API_SECRET')

        endpoint = url.replace(self.base_url, '')
        endpoint = '/v1' + endpoint

        timestamp = int(round(time.time() * 1000))
        nonce = timestamp * 1000 + 1000000000
        logger.debug('Nonce: %s' % nonce)

        try:
            data = request_args['data']
            # payload get passed as a header, so we don't want to POST them
            request_args['data'] = {}
        except KeyError:
            data = {}

        payloadObject = {
            'request':endpoint,
            'nonce':str(nonce),
        }
        payloadObject.update(data)
        payload_json = json.dumps(payloadObject)
        payload = str(base64.b64encode(payload_json))

        sig = hmac.new(self.secret, payload, hashlib.sha384).hexdigest()

        try:
           headers = request_args['headers']
        except KeyError:
           headers = request_args['headers'] = {}

        headers['X-BFX-APIKEY'] = self.api_key
        headers['X-BFX-PAYLOAD'] = payload
        headers['X-BFX-SIGNATURE'] = sig

    def balance_req(self):
        return self.req('get', '/balances')

    def balance_resp(self, req):
        raw_balances = self.resp(req)

        btc_available = None
        usd_available = None
        for raw_balance in raw_balances:
            if raw_balance['type'] == 'exchange':
                if raw_balance['currency'] == 'btc':
                    btc_available = Money(raw_balance['available'], 'BTC')
                elif raw_balance['currency'] == 'usd':
                    usd_available = Money(raw_balance['available'], 'USD')

        if btc_available == None or usd_available == None:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'missing expected balances',
            )

        balance = Balance()
        balance['BTC'] = btc_available
        balance['USD'] = usd_available

        return balance

    def ticker_req(self, verify=True):
        return self.req('get', '/pubticker/btcusd', no_auth=True, verify=verify)

    def ticker_resp(self, req):
        response = self.resp(req)
        return {
            'high': Money(response['high'], 'USD'),
            'low': Money(response['low'], 'USD'),
            'last': Money(response['last_price'], 'USD'),
            'volume': Money(response['volume'], 'BTC')
        }

    def _get_order_book_req(self, verify=True):
        return self.req('get', '/book/btcusd', no_auth=True, verify=verify)

    @property
    def _orderbook_sort_key(self):
        return lambda o: float(o['price'])

    def parse_order(self, order):
        price = Money(order['price'], 'USD')
        volume = Money(order['amount'], 'BTC')
        return (price, volume)

    def create_trade_req(self, mode, volume, price, is_market_order=False):
        side = self.from_const(mode)

        if price.currency != 'USD':
            raise ValueError('price must be in USD')
        if volume.currency != 'BTC':
            raise ValueError('volume must be in BTC')

        volume_str = '%.8f' % volume.amount
        price_str = '%.8f' % price.amount

        payload = {
            'symbol': 'btcusd',
            'amount': volume_str,
            'price': price_str,
            'exchange': 'bitfinex',
            'side': side,
            'type': 'exchange limit',
        }

        if is_market_order == False:
            payload['is_postonly'] = True

        return self.req('post', '/order/new', data=payload)

    def create_trade_resp(self, req):
        response = self.resp(req)
        try:
            order_id = str(response['order_id'])
            return {'success': True, 'order_id': order_id}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order id',
            )

    def open_orders_req(self):
        return self.req('post', '/orders')

    def open_orders_resp(self, req):
        raw_open_orders = self.resp(req)
        open_orders = []

        for raw_order in raw_open_orders:
            mode = self._order_mode_to_const(raw_order['side'])
            volume = Money(raw_order['original_amount'], 'BTC')
            volume_remaining = Money(raw_order['remaining_amount'], 'BTC')

            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': Money(raw_order['price'], 'USD'),
                'volume': volume,
                'volume_remaining': volume_remaining
            }

            open_orders.append(order)

        return open_orders

    def multi_order_details_req(self):
        return self.trades_for_orders_req()

    def multi_order_details_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]

        multi_trades = self.trades_for_orders_resp(req, order_ids)
        data = {}

        for order_id in order_ids:
            total_usd = Money('0', 'USD')
            total_btc = Money('0', 'BTC')
            our_trades = []
            our_type = None

            if order_id in multi_trades:
                trades = multi_trades[order_id]

                for t in trades:
                    btc_amount = Money(t['amount'], 'BTC')
                    fee = abs(Money(t['fee_amount'], t['fee_currency']))
                    price = Money(t['price'], 'USD')

                    usd_amount = price * btc_amount.amount
                    usd_amount = usd_amount.round_to_decimal_places(
                            8,
                            rounding=cdecimal.ROUND_HALF_UP,
                    )

                    our_type = self._order_mode_to_const(t['type'].lower())

                    total_usd += usd_amount
                    total_btc += btc_amount

                    our_trades.append({
                        'time': int(float(t['timestamp'])),
                        'trade_id': unicode(t['tid']),
                        'fee': fee,
                        'btc': btc_amount,
                        'fiat': usd_amount,
                    })

            try:
                time_created = min([t['time'] for t in our_trades])
            except ValueError: # no trades
                time_created = None

            data[order_id] = {
                'time_created': time_created,
                'type': our_type,
                'btc_total': total_btc,
                'fiat_total': total_usd,
                'trades': our_trades
            }

        return data

    def cancel_order_req(self, order_id):
        payload = {
            'order_id': int(order_id)
        }
        return self.req('post', '/order/cancel', data=payload)

    def cancel_order_resp(self, req):
        response = self.resp(req)
        # We want to make sure the order is actually cancelled before going on
        time.sleep(2)
        return {'success': True}

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if not isinstance(volume, Money) or volume.currency != 'BTC':
            raise TypeError('Withdrawal volume must be in BTC')

        payload = {
            'withdraw_type': 'bitcoin',
            'walletselected': 'exchange',
            'amount': str(volume.amount),
            'address': address,
        }

        return self.req('post', '/withdraw', data=payload)

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)

        # not sure why this comes back in an array
        # [{u'status': u'error', u'message': u'Insufficient ...', u'withdrawal_id': 0}]
        response = response[0]

        if response['status'] != 'success':
            raise exceptions.ExchangeAPIErrorException(self, response['message'])

        return {'success': True, 'exchange_withdrawal_id': response['withdrawal_id']}

    # returns an OrderedDict of order ids mapped to their filled volume (only include orders that have some trades)
    def audit(self, skip_recent=0):
        orders = OrderedDict()
        trades_to_audit = self.all_trades()
        # Trades from the same order aren't guaranteed to be next to each other, so we need to group them
        trades_to_audit.sort(key=lambda t:(t['order_id'], t['timestamp']), reverse=True)

        # we want to skip recent orders, but splitting between trades will give us incorrect volume_filled
        orders_to_skip = skip_recent

        while orders_to_skip > 0:
            try:
                trade = trades_to_audit.pop(0)
                # we found a boundary
                if trade['order_id'] != trades_to_audit[0]['order_id']:
                    orders_to_skip -= 1
            except IndexError:
                # if we run off the end of the list, we are trying to strip all trades
                trades_to_audit = []
                orders_to_skip = 0

        for trade in trades_to_audit:
            order_id = str(trade['order_id'])

            try:
                orders[order_id] += abs(Money(trade['amount'], 'BTC'))
            except KeyError:
                orders[order_id] = abs(Money(trade['amount'], 'BTC'))

        # Remove the oldest 2 orders, because its trades might be wrapped around a page
        # gap. This would give us an innacurate volume_filled number. We need to remove
        # 2, since there could be an ask and a bid.
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        min_fee = Money('20', 'USD')
        percentage_fee = deposit_amount * Decimal('0.001') # 0.1%
        return max(min_fee, percentage_fee)

    def fiat_withdrawal_fee(self, withdrawal_amount):
        min_fee = Money('20', 'USD')
        percentage_fee = withdrawal_amount * Decimal('0.0025') # 0.25% for Express Wires
        base_fee = max(min_fee, percentage_fee)

        # This is probably getting added by some intermediary bank between Bitfinex and
        # us.
        extra_fee = Money('21.8', 'USD')

        return base_fee + extra_fee



import os
from sqlalchemy import UnicodeText, TypeDecorator
from gryphon.lib.encrypt import *

try:
    DB_ENCRYPT_KEY = os.environ['DB_ENCRYPT_KEY']
except:
    raise Exception("""Requires environment variables: DB_ENCRYPT_KEY""")

class EncryptedUnicodeText(TypeDecorator):
    impl = UnicodeText

    def process_bind_param(self, value, dialect):
        secret_key = DB_ENCRYPT_KEY
        if value:
            return encrypt(value, secret_key)
        else:
            return value

    def process_result_value(self, value, dialect):
        secret_key = DB_ENCRYPT_KEY
        if value:
            return decrypt(value, secret_key)
        else:
            return value



"""
This library gives two main interface functions right now:
  intraday_volatility_of_day ~ gives an indicator of how frothy trading within a day
    was. This measure is designed to be used to compare days to eachother.
  interday_volatility_of_period ~ gives an indicator of how frothy trading over a
    period of many days is. This could be used to compare months, or feel out what
    price swings in the "current regime" might be like.

Notes on volatility:

We have several different data sources we can calculate volatility of
  i) Our Core FV
  ii) Trade series
  iii) Middle prices from orderbooks

These functions use core fv to start because that is simplest, but we should move to
using the trade series or middle prices sometime soon. There are lots of different
choices for which type of volatility we look at: the period we look back, the sample
frequency, whether to annualize, whether to use returns or log returns, and which
metric to  use. Common metrics are stddev, variance, average absolute return.

Sources for this math:
- Section 3.2.4 of High Frequency Finance (textbook)
- Trading & Exchanges Chapter 20
- https://en.wikipedia.org/wiki/Volatility_(finance)
- http://www.investopedia.com/university/optionvolatility/volatility2.asp?no_header_alt=true
- http://quant.stackexchange.com/questions/2589/how-to-calculate-historical-intraday-volatility

TODO:
- Test methods against some known dataset.
- Move to looking at our saved trade history instead of core fv.
- Doublecheck that the way we annualize daily returns is valid.
"""

import datetime
import math

from cdecimal import Decimal
from delorean import Delorean
import numpy
from sqlalchemy import extract, func

from gryphon.lib.models.datum import Datum


def intraday_volatility_of_day(db, day_start):
    """
    Answers the question: "How volatile was this day". This is the standard deviation
    of hourly log return samples over a period of 24 hours beginning at day_start.

    Performance: very fast.
    """
    day_end = day_start + datetime.timedelta(days=1)

    volatility = hourly_volatility_in_period_from_core_fv_log_returns(
        db,
        day_start,
        day_end,
    )

    return volatility


def interday_volatility_of_period(db, first_day, last_day):
    """
    Answers the question "How bumpy was this period of several days?" The specific
    calculation is the standard deviation of daily log return samples at day end
    (11:59 pm UTC), annualized to a 365 day trading year. We can use this to calculate
    HV10/30/60/150 or arbitrary periods.

    Performance: don't try much more than 15 days at once.
    """

    period_end = last_day + datetime.timedelta(days=1)

    volatility = daily_volatility_in_period_from_core_fv_log_returns(
        db,
        first_day,
        period_end,
    )

    return volatility


def hourly_volatility_in_period_from_core_fv_log_returns(db, start_time, end_time):
    # get core_fv for each hour
    hourly_fv_series = get_hourly_core_fv_series_in_period(db, start_time, end_time)

    # convert to log returns
    log_returns_series = convert_to_log_returns(hourly_fv_series)

    # calculate volatility
    volatility = numpy.std(log_returns_series)

    return volatility


def one_hour_trailing_volatility_10m_buckets_core_fv_log_returns(db, end_time):
    """
    Answers the question "How bumpy was the last hour?"
    """
    # Round up end_time. Rounding up is preferable because this is an open/open
    # series, so rounding up gets us one more datapoint.
    end_time = round_datetime_up_to_10m(end_time)

    start_time = end_time - datetime.timedelta(seconds=3600)

    fv_series = get_ten_minute_core_fv_series_in_period(db, start_time, end_time)

    log_returns_series = convert_to_log_returns(fv_series)

    volatility = numpy.std(log_returns_series)

    return volatility


def three_hour_trailing_volatility_10m_buckets_core_fv_log_returns(db, end_time):
    """
    Answers the question "How bumpy was the last three hours?"
    """

    # Round up end_time. Rounding up is preferable because this is an open/open
    # series, so rounding up gets us one more datapoint.
    end_time = round_datetime_up_to_10m(end_time)

    start_time = end_time - datetime.timedelta(seconds=10800)

    fv_series = get_ten_minute_core_fv_series_in_period(db, start_time, end_time)

    log_returns_series = convert_to_log_returns(fv_series)

    volatility = numpy.std(log_returns_series)

    return volatility


def get_hourly_core_fv_series_in_period(db, start_time, end_time):
    """
    Get the hourly open-open price series in this period from our core fv datums.
    Like most of our periods, this one is inclusive on the left but not the right. So
    getting this series for a full day [Nov 1st, Nov 2nd) will give you 24 datapoints,
    but getting this series for a day and one minute will likely give you 25.
    """

    fundamental_value_series = []

    fv_series = db.query(
            Datum.time_created,
            Datum.numeric_value,
        )\
        .filter(Datum.time_created >= start_time)\
        .filter(Datum.time_created < end_time)\
        .filter(Datum.datum_type.like('%_CORE_FUNDAMENTAL_VALUE'))\
        .group_by(func.date(Datum.time_created), func.hour(Datum.time_created))\
        .all()

    # Do a teeny bit of sanity checking. The fanciness here is because timedelta
    # doesn't have a .hours method.
    delta = end_time - start_time
    hours = 24*delta.days + math.ceil(delta.seconds / 3600.0)
    assert(len(fv_series) == hours)

    fundamental_value_series = convert_datum_series_to_time_series(fv_series)

    return fundamental_value_series


def get_ten_minute_core_fv_series_in_period(db, start_time, end_time):
    """
    Get the ten-minute open-price series in this period from our core fv datums.
    Like most of our periods, this one is inclusive on the left but not the right. So
    getting this series for a full day [Nov 1st, Nov 2nd) will give you 24*6 == 144
    datapoints, but getting this series for a day and one minute will likely give you
    25.
    """

    fundamental_value_series = []

    fv_series = db.query(
            Datum.time_created,
            Datum.numeric_value,
        )\
        .filter(Datum.time_created >= start_time)\
        .filter(Datum.time_created < end_time)\
        .filter(Datum.datum_type.like('%_CORE_FUNDAMENTAL_VALUE'))\
        .group_by(
            func.date(Datum.time_created),
            func.hour(Datum.time_created),
            func.floor((func.minute(Datum.time_created) / 10))*10
        )\
        .all()

    # Do a teeny bit of sanity checking.
    delta = end_time - start_time
    ten_minutes = math.floor((delta.days*86400 + delta.seconds) / 600)
    assert(len(fv_series) == ten_minutes)

    fundamental_value_series = convert_datum_series_to_time_series(fv_series)

    return fundamental_value_series


def daily_volatility_in_period_from_core_fv_log_returns(db, start_time, end_time):
    # get core_fv for each hour
    daily_fv_series = get_daily_core_fv_series_in_period(db, start_time, end_time)

    # convert to log returns
    log_returns_series = convert_to_log_returns(daily_fv_series)

    # calculate volatility
    volatility = numpy.std(log_returns_series)

    # There's a question here of what "annualizing" a bitcoin price means. I decided
    # to use 365 as the constant, because that makes the intuition of the resulting
    # number "The standard deviation of the spread of log returns over a year". It
    # does mean that the number isn't comparable to stocks, which have a 252 day
    # trading year.
    annualized_volatility = volatility * math.sqrt(365)

    return annualized_volatility


def get_daily_core_fv_series_in_period(db, start_time, end_time):
    """
    Get the daily close-close price series in this period from our core fv datums.
    NOTE that we use close prices for this series, not open prices like for hourly.
    Getting this series for the period [Nov 1st, Nov 4th) will give you three
    datapoints.
    """

    fundamental_value_series = []

    # Getting the absolute last core_fv datum in a series of periods efficiently is
    # difficult, so this query gets the first datum in the last five minutes of
    # every day.
    fv_series = db.query(
            Datum.time_created,
            Datum.numeric_value,
        )\
        .filter(Datum.time_created >= start_time)\
        .filter(Datum.time_created < end_time)\
        .filter(func.hour(Datum.time_created) == 23)\
        .filter(func.minute(Datum.time_created) > 55)\
        .filter(Datum.datum_type.like('%_CORE_FUNDAMENTAL_VALUE'))\
        .group_by(func.date(Datum.time_created))\
        .all()

    # Do a teeny bit of sanity checking on the data
    delta = end_time - start_time
    assert(len(fv_series) == delta.days)

    fundamental_value_series = convert_datum_series_to_time_series(fv_series)

    return fundamental_value_series


### Helper functions ###


def convert_to_log_returns(series):
    """
    The logarithmic return between two periods is
    ln(p2 / p1) == ln(p2) - ln(p1)
    """

    # Note that Math.log defaults to the natural logarithm.
    log_price_series = [math.log(float(price)) for time, price in series]
    log_returns_series = numpy.diff([log_price for log_price in log_price_series])

    return log_returns_series


def convert_datum_series_to_time_series(datum_series):
    time_series = []

    for timestamp, value in datum_series:
        timestamp = Delorean(timestamp, "UTC").epoch * 1000

        time_series.append([
            timestamp,
            value,
        ])

    return time_series


def round_datetime_up_to_10m(dt):
    if dt.minute % 10 != 0:
        missing_minutes = 10 - dt.minute % 10
        return dt + datetime.timedelta(minutes=missing_minutes)
    else:
        return dt


# The below functions use ordinary returns [(p2 - p1)/p1] insted of log returns. Just
# including them here for reference and maybe we'll use them somewhere later.


def hourly_volatility_in_period_from_core_fv_ordinary_returns(db, start_time, end_time):
    # get core_fv for each hour
    hourly_fv_series = get_hourly_core_fv_series_in_period(db, start_time, end_time)

    # convert to returns
    returns_series = convert_to_ordinary_returns(hourly_fv_series)

    # calculate volatility
    volatility = numpy.std(returns_series)

    return volatility


def convert_to_ordinary_returns(series):
    price_series = [p[1] for p in series]

    returns_series = []

    for i in range(0, len(price_series) - 1):
        p2 = float(price_series[i+1])
        p1 = float(price_series[i])

        r = (p2 - p1) / p1

        returns_series.append(r)

    return returns_series


### Legacy Functions ###

def volatility(values, timestamps, window_time=10000):
    assert len(values) == len(timestamps)
    vols = []
    values.reverse()
    timestamps.reverse()
    for i in range(len(timestamps)):
        noted_timestamp = timestamps[i]
        relevant_values = []
        relevant_timestamps = []
        for j in range(len(timestamps)):
            index = j+i
            if index > len(timestamps)-1:
                break 
            elif noted_timestamp - timestamps[index] < window_time:
                relevant_values.append(values[index])
                relevant_timestamps.append(noted_timestamp - timestamps[index])
            else:
                break
        
        standard_deviation = np.std(relevant_values)
        mean = np.mean(relevant_values)
        #volatility = Decimal(standard_deviation)/Decimal(mean)
        vols.append(standard_deviation)
    vols.reverse()
    return vols





from delorean import Delorean

from gryphon.data_service.auditors.orderbook_auditor import OrderbookAuditor


class BitfinexOrderbookAuditor(OrderbookAuditor):
    def __init__(self):
        super(BitfinexOrderbookAuditor, self).__init__()
        self.exchange_name = 'BITFINEX'
        self.orderbook_url = 'https://api.bitfinex.com/v1/book/btcusd'
        self.audit_time = 60

        self.acceptable_changes_threshold = 20

    def get_timestamp(self, new_orderbook):
        return Delorean().datetime



"""
https://www.bitstamp.net/api/
"""

# -*- coding: utf-8 -*-
from collections import OrderedDict
import hashlib
import hmac
import time

from cdecimal import *
from delorean import Delorean, epoch
from requests_futures.sessions import FuturesSession
from requests_toolbelt.cookies.forgetful import ForgetfulCookieJar

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)


class BitstampBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        # 2018-4-4: This is a hack for an outstanding issue in our environment that
        # causes bitstamp to reject all but the first requests made with a
        # requests.Session() object. See trello for more information.
        if not session:
            session = FuturesSession(max_workers=10)
            session.cookies = ForgetfulCookieJar()

        super(BitstampBTCUSDExchange, self).__init__(session)

        # Immutable properties.
        # TODO: Check on status of the withdrawal_requests_url (might need a dash).
        # TODO: Check if the withdraw_url is still being used or why it isn't in the
        #   v2 API.
        self.name = u'BITSTAMP_BTC_USD'
        self.friendly_name = u'Bitstamp BTC-USD'
        self.base_url = 'https://www.bitstamp.net/api/v2/'
        self.currency = u'USD'
        self.volume_currency = 'BTC'
        self.price_decimal_precision = 2

        # Configurables defaults.
        self.market_order_fee = self.fee
        self.limit_order_fee = self.fee
        self.fee = Decimal('0.0005')  # TODO: update these.
        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0.001', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

        # Endpoints.
        self.ticker_url = 'ticker/btcusd/'
        self.orderbook_url = 'order_book/btcusd/'
        self.buy_url = 'buy/btcusd/'
        self.sell_url = 'sell/btcusd/'
        self.open_orders_url = 'open_orders/btcusd/'
        self.trade_status_url = 'user_transactions/btcusd/'
        self.balance_url = 'balance/'
        self.trade_cancel_url = 'cancel_order/'
        self.withdrawl_requests_url = 'withdrawal_requests/'
        self.withdraw_url = 'https://priv-api.bitstamp.net/api/bitcoin_withdrawal/'
 
    def resp(self, req):
        response = super(BitstampBTCUSDExchange, self).resp(req)

        try:
            errors = response.get('error', None)
        except AttributeError: # Some endpoints return a list.
            errors = None

        if errors:
            errors_string = str(errors)

            if 'You have only' in errors_string:
                raise exceptions.InsufficientFundsError()
            elif 'Order not found' in errors_string:
                raise exceptions.CancelOrderNotFoundError()
            elif 'Minimum order size' in errors_string:
                raise exceptions.MinimumOrderSizeError()
            elif 'Invalid nonce' in errors_string:
                raise exceptions.NonceError()
            else:
                raise exceptions.ExchangeAPIErrorException(self, errors_string)

        return response

    def get_ticker_req(self, verify=True):
        return self.req('get', self.ticker_url, no_auth=True, verify=verify)

    def get_ticker_resp(self, req):
        response = self.resp(req)

        return {
            'high': Money(response['high'], self.currency),
            'low': Money(response['low'], self.currency),
            'last': Money(response['last'], self.currency),
            'volume': Money(response['volume'], self.volume_currency)
        }

    def order_is_open(self, order_id):
        order_id = unicode(order_id)
        open_orders = self.open_orders()
        matching_orders = [o for o in open_orders if unicode(o.get('id')) == order_id]
        is_open = len(matching_orders) > 0

        return is_open

    def all_transactions(self, page=0):
        req = self.all_transactions_req(page=page)
        return self.all_transactions_resp(req)

    def all_transactions_req(self, page=0):
        offset = page * 100
        payload = {
            'offset': offset,
        }

        return self.req('post', self.trade_status_url, data=payload)

    def all_transactions_resp(self, req):
        return self.resp(req)

    def all_trades(self, page=0):
        req = self.all_trades_req(page=page)
        return self.all_trades_resp(req)

    def all_trades_req(self, page=0):
        return self.all_transactions_req(page=page)

    def all_trades_resp(self, req):
        transactions = self.all_transactions_resp(req)
        trades = [t for t in transactions if t['type'] == '2']

        return trades

    def trades_for_orders(self, order_ids):
        req = self.trades_for_orders_req()
        return self.trades_for_orders_resp(req, order_ids)

    def trades_for_orders_req(self):
        return self.all_trades_req()

    def trades_for_orders_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]
        trades = self.all_trades_resp(req)

        matching_trades = {}

        for trade in trades:
            oid = unicode(trade['order_id'])

            if oid in order_ids:
                if oid not in matching_trades:
                    matching_trades[oid] = []

                matching_trades[oid].append(trade)

        return matching_trades

    ###### Common Exchange Methods ######

    def load_creds(self):
        try:
            self.api_key
            self.secret
            self.client_id
        except AttributeError:
            self.api_key = self._load_env('%s_API_KEY' % self.name)
            self.secret = self._load_env('%s_API_SECRET' % self.name)
            self.client_id = self._load_env('%s_CLIENT_ID' % self.name)

    def auth_request(self, req_method, url, request_args):
        """
        modifies request_args
        """
        self.load_creds()

        try:
            payload = request_args['data']
        except KeyError:
            payload = request_args['data'] = {}

        # TODO: fix nonce collisions
        nonce = unicode(int(round(time.time() * 1000)))
        message = nonce + self.client_id + self.api_key

        sig = hmac.new(
            self.secret,
            msg=message,
            digestmod=hashlib.sha256,
        ).hexdigest().upper()

        payload.update({
            'nonce': nonce,
            'key': self.api_key,
            'signature': sig,
        })

    def get_balance_req(self):
        return self.req('post', self.balance_url)

    def get_balance_resp(self, req):
        response = self.resp(req)
        balance = Balance()

        try:
            vol_currency_key = '%s_available' % self.volume_currency.lower()
            price_currency_key = '%s_available' % self.currency.lower()

            balance[self.volume_currency] = Money(
                response[vol_currency_key],
                self.volume_currency,
            )

            balance[self.currency] = Money(response[price_currency_key], self.currency)
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'Balance missing expected keys',
            )

        return balance

    def _get_orderbook_from_api_req(self, verify=True):
        return self.req('get', self.orderbook_url, no_auth=True, verify=verify)

    def _get_orderbook_from_api_resp(self, req):
        order_book = self.resp(req)

        timestamp = int(order_book['timestamp'])
        now = Delorean()

        if epoch(timestamp) < now.last_minute(10):
            raise exceptions.ExchangeAPIErrorException(
                self, 
                'Orderbook is more than 10 minutes old',
            )

        return order_book

    def place_order_req(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        if mode == Consts.ASK:
            url = self.sell_url
        elif mode == Consts.BID:
            url = self.buy_url
        else:
            raise ValueError('mode must be one of ask/bid')

        # This is required by Bitstamp API.
        price = price.round_to_decimal_places(2) # Max is 7 digits.
        volume = volume.round_to_decimal_places(8) # Max is 8 decimal places.

        try:
            payload = {
                'amount': volume.amount,
                'price': price.amount
            }
        except AttributeError:
            raise TypeError('volume and price must be Money objects')

        return self.req('post', url, data=payload)

    def place_order_resp(self, req):
        response = self.resp(req)

        try:
            return {'success': True, 'order_id': unicode(response['id'])}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order id',
            )

    def get_open_orders_req(self):
        return self.req('post', self.open_orders_url)

    def get_open_orders_resp(self, req):
        raw_open_orders = self.resp(req)
        open_orders = []

        for raw_order in raw_open_orders:
            if raw_order['type'] == '0':
                mode = Consts.BID
            elif raw_order['type'] == '1':
                mode = Consts.ASK

            usd_price = Money(raw_order['price'], self.currency)
            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': usd_price,
                'volume_remaining': Money(raw_order['amount'], self.volume_currency),
            }

            open_orders.append(order)

        return open_orders

    def get_order_details(self, order_id):
        req = self.get_order_details_req()
        return self.get_order_details_resp(req, order_id)

    def get_order_details_req(self):
        return self.get_multi_order_details_req()

    def get_order_details_resp(self, req, order_id):
        return self.get_multi_order_details_resp(req, [order_id])[order_id]

    def get_multi_order_details(self, order_ids):
        req = self.get_multi_order_details_req()
        return self.get_multi_order_details_resp(req, order_ids)

    def get_multi_order_details_req(self):
        return self.trades_for_orders_req()

    def get_multi_order_details_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]

        multi_trades = self.trades_for_orders_resp(req, order_ids)
        data = {}

        for order_id in order_ids:
            total_price_currency = Money('0', self.currency)
            total_volume_currency = Money('0', self.volume_currency)
            our_trades = []
            our_type = None

            price_currency_key = self.currency.lower()
            vol_currency_key = self.volume_currency.lower()

            if order_id in multi_trades:
                trades = multi_trades[order_id]

                for t in trades:
                    volume_currency_amount = abs(
                        Money(t[vol_currency_key], self.volume_currency),
                    )

                    fee = abs(Money(t['fee'], self.currency))

                    price_currency_amount = abs(
                        Money(t[price_currency_key], self.currency),
                    )

                    total_price_currency += price_currency_amount
                    total_volume_currency += volume_currency_amount

                    if Decimal(t[vol_currency_key]) > 0:
                        our_type = Consts.BID
                    else:
                        our_type = Consts.ASK

                    our_trades.append({
                        'time': int(parse(t['datetime']).epoch),
                        'trade_id': unicode(t['id']),
                        'fee': fee,
                        vol_currency_key: volume_currency_amount,
                        'fiat': price_currency_amount,
                    })

            try:
                time_created = min([t['time'] for t in our_trades])
            except ValueError: # This is raised if there are no trades.
                time_created = None

            data[order_id] = {
                'time_created': time_created,
                'type': our_type,
                '%s_total' % vol_currency_key: total_volume_currency,
                'fiat_total': total_price_currency,
                'trades': our_trades,
            }

        return data

    def cancel_order_req(self, order_id):
        payload = {
            'id': order_id,
        }

        return self.req('post', self.trade_cancel_url, data=payload)

    def cancel_order_resp(self, req):
        response = self.resp(req)
        return {'success': True}

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if (not isinstance(volume, Money)
                or volume.currency != self.volume_currency):
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        payload = {
            'amount': volume.amount,
            'address': address,
        }

        return self.req('post', self.withdraw_url, data=payload)

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)
        return {'success': True}

    def get_order_audit_data(self, skip_recent=0):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (including
        only orders that have some trades).
        """
        orders = OrderedDict()
        trades_to_audit = self.all_trades()

        # Trades from the same order aren't guaranteed to be next to each other, so we
        # need to group them.
        trades_to_audit.sort(key=lambda t: (t['order_id'], t['datetime']), reverse=True)

        # We want to skip recent orders, but splitting between transactions will give
        # us incorrect volume_filled.
        orders_to_skip = skip_recent

        while orders_to_skip > 0:
            try:
                trade = trades_to_audit.pop(0)

                # We found a boundary.
                if trade['order_id'] != trades_to_audit[0]['order_id']:
                    orders_to_skip -= 1
            except IndexError:
                # If we run off the end of the list, we are trying to strip all trades.
                trades_to_audit = []
                orders_to_skip = 0

        for transaction in trades_to_audit:
            # If transaction is a trade
            if transaction['type'] == 2:
                order_id = str(transaction['order_id'])

                try:
                    orders[order_id] += abs(
                        Money(transaction[self.vol_currency_key], self.volume_currency),
                    )
                except KeyError:
                    orders[order_id] = abs(
                        Money(transaction[self.vol_currency_key], self.volume_currency),
                    )

        # Remove the two oldest orders, because its trades might be wrapped around a
        # bitstamp page gap, and this would give us an innacurate volume_filled number.
        # We need to remove two because there could be an ask and a bid.
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        min_fee = Money('7.5', self.currency)
        percentage_fee = deposit_amount * Decimal('0.0005') # 0.05%

        # As of September 2016 we're getting charged an extra $15 on deposits. Bitstamp
        # says it isn't them so it's likely an intermediary bank. TODO look into this
        # with BMO.
        intermediary_fee = Money('15', self.currency)

        return max(min_fee, percentage_fee) + intermediary_fee

    def fiat_withdrawal_fee(self, withdrawal_amount):
        min_fee = Money('15', 'USD')
        percentage_fee = withdrawal_amount * Decimal('0.0009') # 0.09%

        return max(min_fee, percentage_fee)



"""
A bare-bones wrapper class that allows us to turn off heartbeating if we don't want it.
"""

import subprocess


class Heartbeat(object):
    def __init__(self, is_active=False):
        self.is_active = is_active

    def heartbeat(self, heartbeat_key):
        if self.is_active is True:
            filename = 'monit/heartbeat/%s.txt' % heartbeat_key
            subprocess.call(['touch', filename])




import pyximport; pyximport.install()

from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.tests.exceptional.exchange_wrappers.auth_methods import ExchangeAuthMethodsTests


class TestBitstampBTCUSDAuthMethods(ExchangeAuthMethodsTests):
    def setUp(self):
        self.exchange = BitstampBTCUSDExchange()



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money


class OkcoinTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'OKCOIN_BTC_USD'
        self.url = 'https://www.okcoin.com/api/trades.do?since=0&ok=1'
        self.poll_time = 1

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade['price'], 'USD'),
                'volume': Money(trade['amount'], 'BTC'),
                'timestamp': trade['date'],
                'trade_id': trade['tid'],
            }

            trades.append(trade)

        return trades



from cdecimal import Decimal

from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money

logger = get_logger(__name__)


class KrakenBTCCADExchange(KrakenBTCEURExchange):
    def __init__(self, session=None, configuration=None):
        super(KrakenBTCCADExchange, self).__init__(session)

        self.name = u'KRAKEN_BTC_CAD'
        self.friendly_name = u'Kraken BTC-CAD'
        self.currency = 'CAD'
        self.price_decimal_precision = 1 # As of Feb 2018

        # Updated by Gareth on 2016-9-20
        self.market_order_fee = Decimal('0.0026')
        self.limit_order_fee = Decimal('0.0016')
        self.fee = self.market_order_fee
        self.fiat_balance_tolerance = Money('0.0001', 'CAD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.min_order_size = Money('0.002', 'BTC')
        self.max_tick_speed = 5
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    def load_creds(self):
        try:
            self.api_key
            self.secret
        except AttributeError:
            self.api_key = self._load_env('KRAKEN_BTC_CAD_API_KEY')
            self.secret = self._load_env('KRAKEN_BTC_CAD_API_SECRET')



from gryphon.data_service.pollers.volume.kraken_volume_poller import KrakenVolume


class KrakenCADVolume(KrakenVolume):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_CAD'
        self.currency = 'CAD'



# -*- coding: utf-8 -*-
from collections import OrderedDict
import hashlib
import hmac
import time

from cdecimal import *
from delorean import Delorean, parse, epoch

from base import *
from exceptions import *
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


class BitstampETHUSDExchange(BitstampBTCUSDExchange):
    def __init__(self, session=None, configuration=None):
        super(BitstampETHUSDExchange, self).__init__(session)

        self.name = u'BITSTAMP_ETH_USD'
        self.friendly_name = u'Bitstamp ETH-USD'
        self.currency = u'USD'
        self.volume_currency = 'ETH'

        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'ETH')
        self.min_order_size = Money('0.001', 'ETH')

        if configuration:
            self.configure(configuration)

        # Endpoints.
        # TODO: Implement deposit/withdrawal urls for bitstamp eth usd.
        self.withdrawl_requests_url = ''
        self.deposit_address_url = ''

        self.ticker_url = 'ticker/ethusd/'
        self.orderbook_url = 'order_book/ethusd/'
        self.buy_url = 'buy/ethusd/'
        self.sell_url = 'sell/ethusd/'
        self.open_orders_url = 'open_orders/ethusd/'
        self.trade_status_url = 'user_transactions/ethusd/'
        self.balance_url = 'balance/'
        self.trade_cancel_url = 'cancel_order/'




from decimal import Decimal

from delorean import Delorean
import sqlalchemy
from sqlalchemy.orm import joinedload
from sqlalchemy import func

from gryphon.lib.models.trade import Trade
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money
from gryphon.dashboards.util.store_calculations import store_calculation_result


def get_all_trades_for_period(gryphon_session, start_time, end_time):
    trades = gryphon_session\
        .query(Trade)\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order').joinedload('datums'))\
        .all()

    return trades


def get_all_latest_order(gryphon_session, end_time):
    latest_order = gryphon_session\
        .query(Order)\
        .filter(Order.time_created < end_time)\
        .order_by(Order.time_created.desc())\
        .first()

    return latest_order


def get_manual_trades_for_period(gryphon_session, start_time, end_time):
    trades = gryphon_session\
        .query(Trade)\
        .join(Order)\
        .filter(Order.actor == 'Manual')\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order').joinedload('datums'))\
        .all()

    return trades


def get_manual_latest_order(gryphon_session, end_time):
    # this logic is weird but this is what was in the gryphon handler
    # we'll revisit it soon

    latest_order = gryphon_session\
        .query(Order)\
        .filter(Order._exchange_name == 'BITSTAMP')\
        .filter(Order.actor != "Manual")\
        .filter(Order.time_created < end_time)\
        .order_by(Order.time_created.desc())\
        .first()

    return latest_order


def get_strategy_trades_for_period(gryphon_session, strategy_name, start_time, end_time):
    trades = gryphon_session\
        .query(Trade)\
        .join(Order)\
        .filter(Order.actor == strategy_name)\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order').joinedload('datums'))\
        .all()

    return trades


def get_strategy_latest_order(gryphon_session, strategy_name, start_time, end_time):
    latest_order = gryphon_session\
        .query(Order)\
        .filter(Order.actor == strategy_name)\
        .filter(Order.time_created >= start_time)\
        .filter(Order.time_created < end_time)\
        .order_by(Order.time_created.desc())\
        .first()

    return latest_order


def get_multi_trades_for_period(gryphon_session, start_time, end_time):
    return get_strategy_trades_for_period(
        gryphon_session,
        'Multi',
        start_time,
        end_time,
    )


def get_multi_latest_order(gryphon_session, end_time):
    return get_strategy_latest_order_for_period(
        gryphon_session,
        'Multi',
        start_time,
        end_time,
    )


def get_exchange_trades_for_period(gryphon_session, exchange, start_time, end_time):
    trades = gryphon_session\
        .query(Trade)\
        .join(Order)\
        .filter(Order._exchange_name == exchange)\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order').joinedload('datums'))\
        .all()

    return trades


def get_exchange_latest_order(gryphon_session, exchange_name, end_time):
    # Exclude Manual trades since they don't have a fundamental value
    latest_order = gryphon_session\
        .query(Order)\
        .filter(Order._exchange_name == exchange_name)\
        .filter(Order.actor != "Manual")\
        .filter(Order.time_created < end_time)\
        .order_by(Order.time_created.desc())\
        .first()

    return latest_order


def get_total_volume_in_period(db, start_time, end_time):
    total_volume = db\
        .query(func.sum(Trade._volume))\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .first()

    return total_volume[0]


def get_total_volume_for_exchange_in_period_from_database(db, start_time, end_time, exchange):
    results = db.query('sum(trade.volume)')\
        .from_statement("select sum(trade.volume) from `order` inner join trade on `order`.order_id=trade.order_id where `order`.time_created < :end_time and `order`.time_created > :start_time and `order`.exchange_name = :exchange")\
        .params(
            start_time=start_time, 
            end_time=end_time, 
            exchange=exchange)\
        .first()

    total_volume = results[0]

    if not total_volume:
        total_volume = Decimal(0)

    return total_volume


def get_last_trade_price_in_usd(db):
    last_trade = db.query(Trade)\
        .order_by(Trade.time_created.desc())\
        .first()

    last_trade_price = last_trade.order.price.to(
        'USD',
        exchange_rate_to_usd=last_trade.order.exchange_rate,
    )

    return last_trade_price


def get_last_fundamental_value_in_usd(db):
    last_order = db.query(Order)\
        .order_by(Order.time_created.desc())\
        .first()

    fundamental_value = last_order.fundamental_value.to(
        'USD',
        exchange_rate_to_usd=last_order.exchange_rate,
    )

    return fundamental_value


def get_open_bids_and_asks(db):
    quotes = db.query(Order)\
        .filter(Order.status == 'OPEN')\
        .filter(Order.time_created > Delorean().last_hour(1).naive())\
        .filter(Order.actor == 'MULTI')\
        .all()

    bids = [q for q in quotes if q.order_type == Order.BID]
    asks = [q for q in quotes if q.order_type == Order.ASK]

    bids = sorted(
        bids, 
        key=lambda quote: quote.price.amount*quote.exchange_rate,
        reverse=True,
    )

    asks = sorted(
        asks, 
        key=lambda quote: quote.price.amount*quote.exchange_rate,
        reverse=True,
    )

    quotes = {
        'bids': bids,
        'asks': asks,
    }

    return quotes


def get_num_active_exchanges_in_period_from_database(db, start_time, end_time):
    column_name = 'num_live_exchanges'

    results = db.query(column_name)\
        .from_statement("select count(distinct `order`.exchange_name) as :column_name from `order` where `order`.time_created > :start_time and `order`.time_created < :end_time")\
        .params(
            column_name=column_name,
            start_time=start_time, 
            end_time=end_time)\
        .first()

    return results[0]

def strip_datetime_for_db_operations(dt):
    """
    Some of our queries get very slow unless we remove the timezone info
    before we put them into sql.
    """
    return dt.replace(tzinfo=None)




import logging

import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.lib.gryphonfury import fees

logger = logging.getLogger(__name__)


class FeesDashboardHandler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self):
        start_time, end_time = self.get_start_time_and_end_time()

        total_trading_fees = fees.get_all_fees_in_period_in_usd(
            self.trading_db,
            start_time,
            end_time,
        )

        fees_by_exchange = fees.get_all_fees_in_period_by_exchange_in_usd(
            self.trading_db,
            start_time,
            end_time,
        )

        total_wire_fees = fees.get_all_wire_fees_in_period_in_usd(
            self.trading_db,
            start_time,
            end_time,
        )

        wire_fees_by_exchange = fees.get_wire_fees_in_period_by_exchange(
            self.trading_db,
            start_time,
            end_time,
        )

        self.render_template(
            'fees.html',
            args={
                'total_trading_fees': total_trading_fees,
                'fees_by_exchange': fees_by_exchange,
                'total_wire_fees': total_wire_fees,
                'wire_fees_by_exchange': wire_fees_by_exchange,
            },
        )




"""
Functions for calculating different versions of the sharpe ratio, defined roughly as the
ratio of the excess returns of an asset to it's standard deviation.

https://en.wikipedia.org/wiki/Sharpe_ratio
"""

import numpy as np
import pandas as pd

def calculate_sharpe_ratio(series, annualize_by_factor=None):
    """
    Given a raw pandas value/prices series calculate the sharpe ratio of the log
    returns with a risk-free rate of 0. This is an unrealistic assumption but we'll
    use it for now.
    """

    # Convert raw prices into log returns.
    log_prices = np.log(series)
    log_returns = log_prices - log_prices.shift(1)

    raw_sharpe = np.mean(log_returns) / np.std(log_returns)

    if annualize_by_factor is not None:
        return raw_sharpe*np.sqrt(annualize_by_factor)
    else:
        return raw_sharpe
